#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

/** Mandatory include files **/

#include "pip_constant.h"
#include "pip_extern.h"

/** Application include files **/

#include "extern.h"


/******************************************************************************
  run_demo_1 ()

  Perform necessary event-checking for panel 1.
******************************************************************************/

run_demo_1 ()
{
  if (Win->update_event)
  {
    /** Timer update event **/
  }
  else
  {
    /** Widget event **/

    check_demo_1_widgets ();
  }
}



/******************************************************************************
  check_demo_1_widgets ()

  Check to see if specific widgets have been flagged by selection events.
******************************************************************************/

check_demo_1_widgets ()
{
  int i;
  char string[100];
  Widget *widget;
  Panel *panel;

  /** Make demo_2_button open second panel and automatically revert	**/
  /** back to 0th state.						**/

  if (check_named_widget_flag (&widget, "demo_2_button"))
  {
    panel = get_named_panel ("demo_2_panel");
    open_panel (panel);

    if (widget->state != 0)
      set_widget_state (widget, 0);
  }

  /** Print demo_1_slider value to stdout **/

  if (check_named_widget_flag (&widget, "demo_1_slider"))
  {
    printf ("%6.4f\n", widget->value);
  }

  /** Copy selected demo_list_text item to demo_text **/

  if (check_named_widget_flag (&widget, "demo_list_text"))
  {
    list_get_entry (widget, string, -1);
    for (i=0; string[i] != ' '; i++);
    string[i] = '\0';
    set_named_widget_label ("demo_text", string);
  }

  /** Make right arrow increment demo_combo_slider **/

  if (check_named_widget_flag (&widget, "demo_combo_1_arrow"))
  {
    widget = get_named_widget ("demo_combo_slider");
    set_widget_value (widget, widget->value + 0.05);
  }

  /** Make left arrow decrement demo_combo_slider **/

  if (check_named_widget_flag (&widget, "demo_combo_2_arrow"))
  {
    widget = get_named_widget ("demo_combo_slider");
    set_widget_value (widget, widget->value - 0.05);
  }

  /** Reset all widget flags (semi-mandatory) **/

  reset_all_flags ();
}



/******************************************************************************
  init_demo_1 ()

  Initialize widgets for panel 1.
******************************************************************************/

init_demo_1 ()
{
  int i;

  Widget *widget;

  char line[100],
       *strings[15],
       *strdup();

  /** Copy global DemoList into array of pointers to strings **/

  for (i=0; i<14; i++)
    strings[i] = strdup (DemoList[i]);

  /** Set list widget **/

  widget = get_named_widget ("demo_list");
  list_load (widget, strings, 14, TRUE, FALSE);

  /** Initialize slider to (possibly) command-line specified value **/

  set_named_widget_value ("demo_4_slider", (double)(Demo->whatever));

  /** Often want to do this to clean things up **/

  check_demo_1_widgets ();
}



/******************************************************************************
  exit_demo_1 ()

  Clean-up widgets when exiting panel 1.
******************************************************************************/

exit_demo_1 ()
{
}
