#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

/** Mandatory include files **/

#include "pip_constant.h"
#include "pip_extern.h"

/** Application include files **/

#include "extern.h"


/******************************************************************************
  startup_application (argc, argv)

  Initialize all global application variables and parse application-specfiic
  command line variables.
******************************************************************************/

startup_application (argc, argv)
  int *argc;
  char **argv;
{
  initialize_application ();

  parse_application (argc, argv);
}



/******************************************************************************
  initialize_application ()

  Allocate and initialize global application variables.
******************************************************************************/

initialize_application ()
{
  Demo = (struct demo_info *)(malloc(sizeof(struct demo_info)));

  Demo->boolean =		FALSE;
  Demo->whatever =		2124;
  strcpy (Demo->filename,	"input.dat");
}




/******************************************************************************
  parse_application (argc, argv)

  Parse the application-specific command line arguments given by <argc> and
  <argv>.  The command line will already be stripped of XPIP commands.
******************************************************************************/

parse_application (argc, argv)
  int *argc;
  char **argv;
{
  int i, index;

  double atof();

  if (is_partial_arg("-help", *argc, argv))
  {
    print_application_help_message ();
    exit (0);
  }

  if (index = is_partial_arg("-i", *argc, argv))
  {
    strcpy (Demo->filename, argv[index+1]);
    erase_argv (argc, argv, index, 2);
  }

  if (index = is_partial_arg("-w", *argc, argv))
  {
    Demo->whatever = atoi (argv[index+1]);
    erase_argv (argc, argv, index, 2);
  }

  if (index = is_partial_arg("-b", *argc, argv))
  {
    Demo->boolean = TRUE;
    erase_argv (argc, argv, index, 1);
  }
}




/******************************************************************************
  print_application_help_message ()

  Display application-specific help message.
******************************************************************************/

print_application_help_message ()
{
  printf ("  -input         <string>   demo filename\n");
  printf ("  -w             <int>      set value of whatever\n");
  printf ("  -b                        set boolean\n");
  printf ("\n");
}
