#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

/** Mandatory include files **/

#include "pip_constant.h"
#include "pip_extern.h"

/** Application include files **/

#include "global.h"

/** Define the panel index **/

#define PANEL_DEMO	0


/******************************************************************************
  run_buffer (panel_index, event)

  Interface XPIP with the application by running panel-specific routines
  (as indicated by <panel_index>.   This routine is called whenever there is
  any sort of input event, even if that event doesn't necessarily flag a
  widget (such as a mouse movement in a button).
******************************************************************************/

run_buffer (panel_index, event)
  int panel_index;
  XEvent event;
{
  /** Process dialog panel events **/

  if (dialog_run (panel_index))
    return;

  /** Process all other panel events **/

  switch (panel_index)
  {
    case PANEL_DEMO:
      demo_run ();
      break;
  }
}



/******************************************************************************
  startup_buffer (argc, argv)

  Initialize the application by setting global variables and parsing what's
  left of the command line after XPIP finishes its parsing.
******************************************************************************/

startup_buffer (argc, argv)
  int *argc;
  char **argv;
{
  startup_application (argc, argv);
}



/******************************************************************************
  init_buffer (panel_index)

  Perform panel-specific initialization (as indicated by <panel_index>).
  Called whenever panel is opened.
******************************************************************************/

init_buffer (panel_index)
  int panel_index;
{
  /** Process dialog panel events **/

  if (dialog_init (panel_index))
    return;

  /** Process all other panel events **/

  switch (panel_index)
  {
    case PANEL_DEMO:
      demo_init ();
      break;
  }
}



/******************************************************************************
  exit_buffer (panel_index)

  Perform panel-specific exit routine (as indicated by <panel_index>).  Called
  whenever panel is closed.
******************************************************************************/

exit_buffer (panel_index)
  int panel_index;
{
  /** Process dialog panel events **/

  if (dialog_exit (panel_index))
    return;

  /** Process all other panel events **/

  switch (panel_index)
  {
    case PANEL_DEMO:
      demo_exit ();
      break;
  }
}
