#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

/** Mandatory include files **/

#include "pip_constant.h"
#include "pip_extern.h"

/** Application include files **/

#include "extern.h"


/******************************************************************************
  demo_run ()

  Perform necessary event-checking.
******************************************************************************/

demo_run ()
{
  if (Win->update_event)
  {
    /** Timer update event **/
  }
  else
  {
    /** Widget event **/

    demo_check_widgets ();
  }
}



/******************************************************************************
  demo_check_widgets ()

  Check to see if specific widgets have been flagged by selection events.
******************************************************************************/

demo_check_widgets ()
{
  int i;
  char string[100];
  Widget *widget;
  Panel *panel;

  if (check_named_widget_flag (&widget, "demo_load_button"))
  {
    if (widget->state == 1)
    {
      file_init_widgets (Demo->directory, "*.lst");
      file_set_callback_widget (widget);
      open_panel (get_named_panel ("file_panel"));
    }
    else if (widget->state == 2) 
    {
      if (widget->user)
      {
        file_get_filename (Demo->pathname, Demo->filename, Demo->directory);
        set_named_widget_label ("demo_file_data", Demo->filename);
        demo_load_file (Demo->pathname);
      }  
      set_widget_state (widget, 0);
    }
  }                            

  if (check_named_widget_flag (&widget, "demo_browse_load_button"))
  {
    if (widget->state == 1)
    {
      file_init_widgets (Demo->browse_directory, "*.c");
      file_set_callback_widget (widget);
      open_panel (get_named_panel ("file_panel"));
    }
    else if (widget->state == 2) 
    {
      if (widget->user)
      {
        file_get_filename (Demo->browse_pathname,
			Demo->browse_filename, Demo->browse_directory);
        set_named_widget_label ("demo_browse_file_data", Demo->browse_filename);
      }  
      set_widget_state (widget, 0);
    }
  }

  if (check_named_widget_flag (&widget, "demo_browse_button"))
  {
    browse_show_file (Demo->browse_filename, Demo->browse_pathname, NULL);
  }

  if (check_named_widget_flag (&widget, "demo_message_button"))
    message_show ("INFORMATION", "This is a information message");
  if (check_named_widget_flag (&widget, "demo_warning_button"))
    message_show_warning ("This is a warning message");
  if (check_named_widget_flag (&widget, "demo_error_button"))
    message_show_error ("This is an error message");

  /** Reset all widget flags (semi-mandatory) **/

  reset_all_flags ();
}



/******************************************************************************
  demo_load_file (pathname)

  Load the contents of the file specified by <pathname> into the LIST widget.
******************************************************************************/

demo_load_file (pathname)
  char *pathname;
{
  int i,
      count;
  char line[DEMO_MAX_STRING_LENGTH],
       *strings[DEMO_MAX_STRING_COUNT],
       *strdup();
  FILE *fp;

  if ((fp = fopen (pathname, "r")) == NULL)
  {
    throw_visual_warning ("Cannot open file %s", pathname);
    return;
  }

  for (count=0; fgets (line, DEMO_MAX_STRING_LENGTH, fp) != NULL &&
				count<DEMO_MAX_STRING_COUNT; count++)
    strings[count] = strdup (line);

  list_load (get_named_widget ("demo_list"), strings, count, TRUE, FALSE);

  for (i=0; i<count; i++)
    free (strings[i]);

  fclose (fp);
}



/******************************************************************************
  demo_init ()

  Initialize widgets.
******************************************************************************/

demo_init ()
{
  set_named_widget_label ("demo_file_data", Demo->filename);
  demo_load_file (Demo->pathname);
}



/******************************************************************************
  demo_exit ()

  Clean-up widgets when exiting.
******************************************************************************/

demo_exit ()
{
}
