#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

/** Mandatory include files **/

#include "pip_constant.h"
#include "pip_extern.h"

/** Application include files **/

#include "extern.h"


/******************************************************************************
  startup_application (argc, argv)

  Initialize all global application variables and parse application-specfiic
  command line variables.
******************************************************************************/

startup_application (argc, argv)
  int *argc;
  char **argv;
{
  initialize_application ();

  parse_application (argc, argv);
}



/******************************************************************************
  initialize_application ()

  Allocate and initialize global application variables.
******************************************************************************/

initialize_application ()
{
  Demo = (struct demo_info *)(malloc(sizeof(struct demo_info)));

  strcpy (Demo->filename,	DEFAULT_DEMO_FILENAME);
  strcpy (Demo->directory,	DEFAULT_DEMO_DIRECTORY);
  sprintf (Demo->pathname, "%s%s", Demo->directory, Demo->filename);

  strcpy (Demo->browse_filename,	DEFAULT_DEMO_BROWSE_FILENAME);
  strcpy (Demo->browse_directory,	DEFAULT_DEMO_BROWSE_DIRECTORY);
  sprintf (Demo->browse_pathname, "%s%s",
		Demo->browse_directory, Demo->browse_filename);
}




/******************************************************************************
  parse_application (argc, argv)

  Parse the application-specific command line arguments given by <argc> and
  <argv>.  The command line will already be stripped of XPIP commands.
******************************************************************************/

parse_application (argc, argv)
  int *argc;
  char **argv;
{
  int i, index;

  double atof();

  if (is_partial_arg("-help", *argc, argv))
  {
    print_application_help_message ();
    exit (0);
  }

  if (index = is_partial_arg("-i", *argc, argv))
  {
    strcpy (Demo->filename, argv[index+1]);
    sprintf (Demo->pathname, "%s%s", Demo->directory, Demo->filename);
    erase_argv (argc, argv, index, 2);
  }

  if (index = is_partial_arg("-d", *argc, argv))
  {
    strcpy (Demo->directory, argv[index+1]);
    sprintf (Demo->pathname, "%s%s", Demo->directory, Demo->filename);
    erase_argv (argc, argv, index, 2);
  }
}




/******************************************************************************
  print_application_help_message ()

  Display application-specific help message.
******************************************************************************/

print_application_help_message ()
{
  printf ("  -input         <string>   default demo filename\n");
  printf ("  -directory     <string>   default demo directory\n");
  printf ("\n");
}
