#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include "pip_constant.h"
#include "pip_extern.h"
#include "pip_generic.h"

#include "pip_c_base.h"


/******************************************************************************
  base_function (widget, line, function)

  Run the type-specific routine for widget <widget> that is associated with
  command <function>, using supplemental information from string <line>.
******************************************************************************/

base_function (widget, line, function)
  Widget *widget;
  char *line;
  short function;
{
  switch (function)
  {
    case FUNCTION_HELP:
      base_help (Console->formats[CONTROL_BASE]);
      break;

    case FUNCTION_FORMAT:
      base_format (Console->formats[CONTROL_BASE]);
      break;

    case FUNCTION_LOAD:
      parse_format_defaults (Console->formats[CONTROL_BASE], line);
      break;

    case FUNCTION_STORE:
      break;

    case FUNCTION_DUMP:
      generic_dump (widget);
      break;

    case FUNCTION_INITIALIZE:
      base_initialize (widget);
      break;

    case FUNCTION_PARSE:
      base_parse (widget, line);
      break;

    case FUNCTION_CLEAN:
      break;

    case FUNCTION_SETUP:
      base_setup (widget);
      break;

    case FUNCTION_EDIT:
      break;

    case FUNCTION_SELECT:
      break;

    case FUNCTION_MOTION:
      break;
 
    case FUNCTION_RELEASE:
      break;

    case FUNCTION_EXECUTE:
      break;

    case FUNCTION_DRAW:
      base_draw (widget);
      break;

    default:
      throw_warning ("Function unknown to base", NULL);
      break;
  }
}



/******************************************************************************
  base_help (format)

  Print a help message to stdout detailing the usage of any type-specific
  arguments for the widget <format>, including default values when possible.
******************************************************************************/

base_help (format)
  Widget *format;
{
  printf ("\n");
  printf ("BASE\n");
  printf ("    COLOR   <index>                base color index\n");
  printf ("\n");
}



/******************************************************************************
  base_format (format)

  Set type-specific default for canonical widget <format>.
******************************************************************************/

base_format (format)
  Widget *format;
{
  format->status[IS_SELECTABLE] =	FALSE;
  format->status[IS_ACTIVATABLE] =	FALSE;
  format->status[IS_EDITABLE] =		EDITABLE_NEVER;
  format->status[IS_MOTION_ACTIVE] =	FALSE;
  format->status[IS_BUFFERABLE] =	BUFFER_NORMAL;

  format->size.x =			DIMENSION_PANEL;
  format->size.y =			DIMENSION_PANEL;
  format->spacing.x =			DEFAULT_BASE_SPACING_X;
  format->spacing.y =			DEFAULT_BASE_SPACING_Y;

  format->thickness[THICKNESS_SHADOW] = 0;
  format->thickness[THICKNESS_ACTIVE] = 0;

  format->gcs[GC_BODY] =		COLOR_CTRL_BACK;;

  format->format =			CONTROL_BASE;
}



/******************************************************************************
  base_initialize (widget)

  Initialize widget <widget> with non-canonical information.
******************************************************************************/

base_initialize (widget)
  Widget *widget;
{
  widget->format = CONTROL_BASE;
}



/******************************************************************************
  base_parse (widget, line)

  Parse any type-specific commands in string <line> for widget <widget>,
  setting internal values as needed.
******************************************************************************/

base_parse (widget, line)
  Widget *widget;
  char *line;
{
  int i, j,
      int_factor;

  char aux_line[100],
       *ptr,
       ch;  

  double factor,
         atof();

  strcpy (aux_line, line);
  strip_first (aux_line);

  factor = atof (aux_line);
  int_factor = atoi (aux_line);
 
  if (line[0] == 'A')
  {
    switch (aux_line[0])
    {
      /** COLOR **/

      case 'C':

        widget->gcs[GC_BODY] = int_factor;
        break;
    }

    line[0] = '#';
  }
}



/******************************************************************************
  base_setup (widget)

  Set up derived parameters (those not directly accessible to user) for
  widget <widget>.
******************************************************************************/

base_setup (widget)
  Widget *widget;
{
  widget->bound_offset.x = 0;
  widget->bound_offset.y = 0;
  widget->bound_size.x = 2*widget->bound_offset.x + widget->size.x;  
  widget->bound_size.y = 2*widget->bound_offset.y + widget->size.y;     
}



/******************************************************************************
  base_draw (widget)

  Draw the widget <widget>.
******************************************************************************/

base_draw (widget)
  Widget *widget;
{
  XFillRectangle (Win->display, widget->drawable,
		Win->hue_widget[widget->gcs[GC_BODY]],
		0, 0, widget->size.x, widget->size.y);
}
