#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include "pip_constant.h"
#include "pip_extern.h"
#include "pip_generic.h"

#include "pip_c_data.h"


/******************************************************************************
  data_function (widget, line, function)

  Run the type-specific routine for widget <widget> that is associated with
  command <function>, using supplemental information from string <line>.
******************************************************************************/

data_function (widget, line, function)
  Widget *widget;
  char *line;
  short function;
{
  switch (function)
  {
    case FUNCTION_HELP:
      data_help (Console->formats[CONTROL_DATA]);
      break;

    case FUNCTION_FORMAT:
      data_format (Console->formats[CONTROL_DATA]);
      break;

    case FUNCTION_LOAD:
      parse_format_defaults (Console->formats[CONTROL_DATA], line);
      break;

    case FUNCTION_STORE:
      break;

    case FUNCTION_DUMP:
      generic_dump (widget);
      break;

    case FUNCTION_INITIALIZE:
      data_initialize (widget);
      break;

    case FUNCTION_PARSE:
      data_parse (widget, line);
      break;

    case FUNCTION_CLEAN:
      break;

    case FUNCTION_SETUP:
      data_setup (widget);
      break;

    case FUNCTION_EDIT:
      data_edit (widget, line);
      break;

    case FUNCTION_SELECT:
      data_select (widget);
      break;

    case FUNCTION_MOTION:
      data_motion (widget);
      break;

    case FUNCTION_RELEASE:
      break;

    case FUNCTION_EXECUTE:
      data_execute (widget);
      break;

    case FUNCTION_DRAW:
      data_draw (widget);
      break;

    default:
      throw_warning ("Function unknown to data", NULL);
      break;
  }
}



/******************************************************************************
  data_help (format)

  Print a help message to stdout detailing the usage of any type-specific
  arguments for the widget <format>, including default values when possible.
******************************************************************************/

data_help (format)
  Widget *format;
{
  printf ("\n");
  printf ("DATA\n");
  printf ("    STRING <string>                text/data field label\n");
  printf ("    TEXT   <string>                text field content\n");
  printf ("    INIT   <float>                 text field content\n");
  printf ("    OUTLINE                        outline rather than indent\n");
  printf ("    ALTER   EDIT                   allow special editing\n");
  printf ("    ALTER   DIGIT   <char>         formating of digits (N)\n",
                        format->data[DATA_D_DIGIT]);
  printf ("    ALTER   TEXT    <char>         formating of text (C)\n",
                        format->data[DATA_D_DIGIT]);
  printf ("    ALTER   LINES   <int>          number of lines (1)\n");
  printf ("\n");
}



/******************************************************************************
  data_format (format)

  Set type-specific default for canonical widget <format>.
******************************************************************************/

data_format (format)
  Widget *format;
{
  format->status[IS_SELECTABLE] =	TRUE;
  format->status[IS_ACTIVATABLE] =	TRUE;
  format->status[IS_EDITABLE] =		EDITABLE_FULL;
  format->status[IS_MOTION_ACTIVE] =	FALSE;
  format->status[IS_BUFFERABLE] =	BUFFER_NORMAL;

  format->size.x =			DEFAULT_DATA_DIMENSION_X;
  format->size.y =			DEFAULT_DATA_DIMENSION_Y;
  format->spacing.x =			DEFAULT_DATA_SPACING_X;
  format->spacing.y =			DEFAULT_DATA_SPACING_Y;

  format->thickness[THICKNESS_SHADOW] =	2;
  format->thickness[THICKNESS_ACTIVE] =	2;

  format->data[DATA_D_DIGIT] =		DEFAULT_DATA_DIGIT;
  format->data[DATA_D_TEXT] =		DEFAULT_DATA_TEXT;
  format->data[DATA_D_LINE_COUNT] =	DEFAULT_DATA_LINE_COUNT;
  format->data[DATA_D_DIGIT_DECIMAL] =	DEFAULT_DATA_DIGIT_DECIMAL;
  format->data[DATA_D_DIGIT_INTEGER] =	DEFAULT_DATA_DIGIT_INTEGER;
  format->data[DATA_D_OUTLINE] =	DEFAULT_DATA_OUTLINE;
  format->data[DATA_D_X_OFFSET] =	DEFAULT_DATA_X_OFFSET;
  format->data[DATA_D_SPECIAL_EDIT] =	DEFAULT_DATA_SPECIAL_EDIT;

  format->gcs[GC_BODY] =		COLOR_CTRL_SURFACE_1;
  format->gcs[GC_BODY_INV] =		COLOR_CTRL_SURFACE_INV;
  format->gcs[GC_FONT_FIRST] =		COLOR_CTRL_TEXT_1;
  format->gcs[GC_FONT_SECOND] =		COLOR_CTRL_DARK;
  format->gcs[DATA_G_SELECT_FORE] =	COLOR_CTRL_AUX_3;
  format->gcs[DATA_G_SELECT_BACK] =	COLOR_CTRL_DARK;

  format->fonts[FONT_FIRST] =		FONT_MEDIUM;
  format->fonts[FONT_SECOND] =		FONT_MEDIUM;

  format->locate =			LABEL_WEST;

  format->format =			CONTROL_DATA;
}



/******************************************************************************
  data_initialize (widget)

  Initialize widget <widget> with non-canonical information.
******************************************************************************/

data_initialize (widget)
  Widget *widget;
{
  widget->format = CONTROL_DATA;
}



/******************************************************************************
  data_parse (widget, line)

  Parse any type-specific commands in string <line> for widget <widget>,
  setting internal values as needed.
******************************************************************************/

data_parse (widget, line)
  Widget *widget;
  char *line;
{
  int i, j,
      int_factor;
 
  char dos_line[100],
       tres_line[100],
       *ptr,
       ch;
 
  double factor,
         atof();
 
  strcpy (dos_line, line);
  strip_first (dos_line);
 
  strcpy (tres_line, dos_line);
  strip_first (tres_line);
 
  factor = atof (tres_line);
  int_factor = atoi (tres_line);

  if (line[0] == 'A')
  {
    switch (dos_line[0])
    {
      /** DIGIT **/

      case 'D':

        if (tres_line[0] >= '0' && tres_line[0] <= '9') 
        {
          widget->data[DATA_D_DIGIT_INTEGER] =	tres_line[0] - '0';
          widget->data[DATA_D_DIGIT_DECIMAL] =	tres_line[2] - '0';
          widget->data[DATA_D_DIGIT] =		'Z';
        }
        else
          widget->data[DATA_D_DIGIT] =		tres_line[0];
        break;

      /** OUTLINE **/

      case 'O':

        widget->data[DATA_D_OUTLINE] =		TRUE;
        break;

      /** EDIT **/

      case 'E':

        widget->data[DATA_D_SPECIAL_EDIT] =	TRUE;
        widget->status[IS_MOTION_ACTIVE] =	MOTION_UPDATE;
        break;

      /** TEXT **/

      case 'T':

        widget->data[DATA_D_TEXT] =		tres_line[0];
        break;

      /** FOREGROUND **/

      case 'F':

        widget->data[DATA_G_SELECT_FORE] =	atoi (tres_line[0]);
        break;

      /** BACKGROUND **/

      case 'B':

        widget->data[DATA_G_SELECT_BACK] =	atoi (tres_line[0]);
        break;

      /** LINES **/

      case 'L':

        widget->data[DATA_D_LINE_COUNT] =	atoi (tres_line[0]);
        break;
    }

    line[0] = '#';
  }
}



/******************************************************************************
  data_setup (widget)

  Set up derived parameters (those not directly accessible to user) for
  widget <widget>.
******************************************************************************/

data_setup (widget)
  Widget *widget;
{
  int i,
      junk,
      yoffset;

  XPoint text_start,
         text_size;
  XCharStruct text_information;

  /** If displaying a number **/

  if (widget->data[DATA_D_DIGIT] != 'N')
  {
    if (widget->data[DATA_D_DIGIT] != 'Z')
      throw_error ("Old ALTER DIGIT format no longer supported", NULL);

    /** Set label using formatting specification **/

    data_set_label (widget, widget->data[DATA_D_DIGIT_INTEGER],
		widget->data[DATA_D_DIGIT_DECIMAL], TRUE);

    /** Center label in widget **/

    center_text (0, 0, widget->size.x, widget->size.y, widget->label,
		&(widget->text_offset[0].x), &(widget->text_offset[0].y),
		widget->fonts[FONT_FIRST]);
  }

  /** Else displaying text **/

  else
  {
    /** If displaying a single line of text **/

    if (widget->data[DATA_D_LINE_COUNT] == 1)
    {
      if (widget->label)
        center_text (0, 0, widget->size.x, widget->size.y, widget->label,
		&(widget->text_offset[0].x), &(widget->text_offset[0].y),
		widget->fonts[FONT_FIRST]);
      else
        center_text (0, 0, widget->size.x, widget->size.y, " ",
		&(widget->text_offset[0].x), &(widget->text_offset[0].y),
		widget->fonts[FONT_FIRST]);

      if (widget->data[DATA_D_TEXT] == 'L')
        widget->text_offset[0].x = DATA_LEFT_OFFSET;
    }

    /** Else displaying a list of text **/

    else
    {
      yoffset = (int)(widget->data_float[DATA_DF_LINE_SPACING]*
			Win->fonts[widget->fonts[FONT_FIRST]]->dimension.y);

      widget->text_offset[0].x = DATA_LEFT_OFFSET;
      widget->text_offset[0].y = DATA_TOP_OFFSET;

      widget->text_offset[1].y = yoffset;
    }
  }

  widget->next_state = -1;

  /** Position descriptive string for widget **/

  if (widget->string == NULL)
    widget->locate = LABEL_NONE;
  else
    locate_text (widget, widget->string, 1, FONT_SECOND, FALSE);

  /** Set widget label **/

  data_set_label (widget, widget->data[DATA_D_DIGIT_INTEGER],
		widget->data[DATA_D_DIGIT_DECIMAL], FALSE);

  /** Set default widget bounds **/

  widget->bound_offset.x = widget->thickness[THICKNESS_SHADOW] +
		widget->thickness[THICKNESS_ACTIVE];
  widget->bound_offset.y = widget->thickness[THICKNESS_SHADOW] +
		widget->thickness[THICKNESS_ACTIVE];
  widget->bound_size.x = 2*widget->bound_offset.x + widget->size.x; 
  widget->bound_size.y = 2*widget->bound_offset.y + widget->size.y;    

  if (widget->locate != LABEL_NONE)
  {
    XTextExtents (Win->fonts[widget->fonts[FONT_SECOND]]->font,
			widget->string, strlen (widget->string),
			&junk, &junk, &junk, &text_information);

    text_start.x = widget->text_offset[1].x - text_information.lbearing;
    text_start.y = widget->text_offset[1].y - text_information.ascent;
    text_size.x = text_information.lbearing + text_information.rbearing;
    text_size.y = text_information.ascent + text_information.descent;

    if (text_start.x < 0)
    {
      widget->bound_offset.x = -text_start.x;
      widget->bound_size.x = widget->bound_offset.x + widget->size.x +
		widget->thickness[THICKNESS_SHADOW] +
		widget->thickness[THICKNESS_ACTIVE];
    }
    else if (text_start.x > widget->size.x)
    {
      widget->bound_size.x = widget->bound_offset.x +
		text_start.x + text_size.x;
    }

    if (text_start.y < 0)
    {
      widget->bound_offset.y = -text_start.y;
      widget->bound_size.y = widget->bound_offset.y + widget->size.y +
		widget->thickness[THICKNESS_SHADOW] +
		widget->thickness[THICKNESS_ACTIVE];
    }
    else if (text_start.y > widget->size.y)
    {
      widget->bound_size.y = widget->bound_offset.y +
		text_start.y + text_size.y;
    }

    widget->status[IS_BUFFERABLE] = BUFFER_CLEAR;
  }

  if (widget->data[DATA_D_SPECIAL_EDIT])
  { 
    if (!Win->fonts[widget->fonts[FONT_FIRST]]->is_fixed_width)
    {
      widget->data[DATA_D_SPECIAL_EDIT] = FALSE;
      widget->status[IS_MOTION_ACTIVE] = FALSE;
      throw_warning ("Cannot use ALTER EDIT with variable pitch font", NULL);
    }
  }
}



/******************************************************************************
  data_set_label (widget, integer, decimal, use_filler)

  Given the formatting instructions specified by <integer> and <decimal>,
  set the label of widget <widget> to <widget->value>.  If <use_filler>,
  then just set a dummy value (for positioning purposes).  Return the label
  string in <string>.
******************************************************************************/

data_set_label (widget, integer, decimal, use_filler)
  Widget *widget;
  int integer,
      decimal,
      use_filler;
{
  int i;
  double value,
         factor;
  char format_string[20],
       string[200];

  if (widget->data[DATA_D_DIGIT] == 'N')
  {
    return;
  }

  if (integer < 0 || decimal < 0)
  {
    set_label (widget, NULL);
    return;
  }

  /** If use_filler, insert dummy value of maximum formatted width **/

  if (use_filler)
  {
    value = 0.0;
    factor = 1.0;
    for (i=0; i<integer; i++)
    {
      value += 8*factor;
      factor *= 10;
    }
    factor = 0.1;
    for (i=0; i<decimal; i++)
    {
      value += 8*factor;
      factor *= 0.1;
    }
  }
  else
    value = widget->value;

  if (decimal == 0)
  {
    sprintf (format_string, "%%%dd", integer);
    sprintf (string, format_string, (int)(value));
  }
  else
  {
    sprintf (format_string, "%%%d.%df", integer+decimal+1, decimal);
    sprintf (string, format_string, value);
  }

  set_label (widget, string);
}



/******************************************************************************
  data_edit (widget, line)

  Manage editing of widget <widget>, based on input string <line>.  The
  internal value <widget->reserve> determines editing status.
******************************************************************************/

data_edit (widget, line)
  Widget *widget;
  char *line;
{
  double atof();

  static char line_start[200],
              line_finish[200],
              line_full[200];

  switch (widget->reserve)
  {
    case EDIT_BEGIN:
      widget->reserve = EDIT_PROCESS;
      if (widget->states[1])
      {
        strcpy (line_start, widget->label);
        line_start[widget->states[0]] = '\0';
        strcpy (line_finish,
			&widget->label[widget->states[0] + widget->states[1]]);
        line_start[widget->states[0]] = '\0';
        widget->states[1] = 0;
      }
      else
      {
        line_start[0] = '\0';
        line_finish[0] = '\0';
      }
      edit_text (widget);
      break;

    case EDIT_PROCESS:
      sprintf (line_full, "%s%s%s", line_start, line, line_finish);
      set_label (widget, line_full);
      if (line_start[0])
        widget->states[1] = strlen (line);
      if (widget->data[DATA_D_DIGIT] == 'N')
      {
        if (widget->data[DATA_D_TEXT] == 'C')
          center_text (0, 0, widget->size.x, widget->size.y, line_full,
		&(widget->text_offset[0].x), &(widget->text_offset[0].y),
		widget->fonts[FONT_FIRST]);
      }
      break;

    case EDIT_FINISH:
      sprintf (line_full, "%s%s%s", line_start, line, line_finish);
      set_label (widget, line_full);
      widget->states[1] = 0;
      if (widget->data[DATA_D_DIGIT] == 'N')
      {
        if (widget->data[DATA_D_TEXT] == 'C')
          center_text (0, 0, widget->size.x, widget->size.y, line_full,
		&(widget->text_offset[0].x), &(widget->text_offset[0].y),
		widget->fonts[FONT_FIRST]);
      }
      else
      {
        widget->value = atof (line_full);
        data_set_label (widget, widget->data[DATA_D_DIGIT_INTEGER],
		widget->data[DATA_D_DIGIT_DECIMAL], FALSE);
      }
      widget->reserve = EDIT_BEGIN;
      widget->flag = TRUE;
      break;
  }
}



/******************************************************************************
  data_select (widget)

  Handle type-specific function of widget <widget> when it is first selected
  by the user.
******************************************************************************/

data_select (widget)
  Widget *widget;
{
  int i,
      mouse,
      y_offset;

  /** If there is more than one text line, determine which line **/

  if (widget->data[DATA_D_LINE_COUNT] > 1)
  {
    mouse = Win->mouse.y - widget->position.y;

    for (i=0; i<widget->data[DATA_D_LINE_COUNT]-1; i++)
    {
      y_offset = widget->text_offset[0].y +
		widget->text_offset[1].y * (i+1);
      if (mouse < y_offset)
        break;
    }

    /** Set next_state to indicate selected line **/

    widget->next_state = widget->state + i;
    if (widget->next_state >= widget->state_count)
      widget->next_state = widget->state_count - 1;
  }
  else if (widget->data[DATA_D_SPECIAL_EDIT])
  {
    mouse = Win->mouse.x - (widget->position.x + widget->text_offset[0].x);

    if (mouse < 0)
      widget->states[0] = 0;
    else
    {
      widget->states[0] = mouse/
		Win->fonts[widget->fonts[FONT_FIRST]]->dimension.x;
      if (widget->states[0] > strlen (widget->label))
        widget->states[0] = strlen (widget->label)-1;
    }

    widget->states[1] = 1;
    widget->states[2] = widget->states[0];
  }
}



/******************************************************************************
  data_motion (widget)

  Handle type-specific function of widget <widget> when mouse moves within
  widget after initial selection.
******************************************************************************/

data_motion (widget)
  Widget *widget;
{
  int mouse,
      position;

  if (widget->data[DATA_D_LINE_COUNT] > 1)
    return;

  if (!widget->data[DATA_D_SPECIAL_EDIT])
    return;

  mouse = Win->mouse.x - (widget->position.x + widget->text_offset[0].x);

  if (mouse < 0)
    mouse = 0;

  position = mouse/Win->fonts[widget->fonts[FONT_FIRST]]->dimension.x;

  if (position < 0)
    position = 0;

  if (position < widget->states[2])
  {
    widget->states[1] = widget->states[2] - position + 1;
    widget->states[0] = position;
  }
  else
  {
    widget->states[0] = widget->states[2];
    widget->states[1] = position - widget->states[2] + 1;
  }

  if (widget->states[0] + widget->states[1] >= strlen (widget->label))
    widget->states[1] = strlen (widget->label) - widget->states[0];
}




/******************************************************************************
  data_execute (widget)

  Execute the function of widget <widget>, as when mouse button is released
  or mouse motion occurs.
******************************************************************************/

data_execute (widget)
  Widget *widget;
{
  widget->flag = TRUE;
}



/******************************************************************************
  data_draw (widget)

  Draw the widget <widget>.
******************************************************************************/

data_draw (widget)
  Widget *widget;
{
  short i,
        color_font_0, color_font_1,
        font_0, font_1,
        line_count;

  char	string[500];

  GC temp_gc;

  /** Draw something different if editing **/

  if (widget->reserve == EDIT_BEGIN)
  {
    if (widget->data[DATA_D_DIGIT] != 'N')
      data_set_label (widget, widget->data[DATA_D_DIGIT_INTEGER],
		widget->data[DATA_D_DIGIT_DECIMAL], FALSE);
    else if (widget->data[DATA_D_TEXT] == 'L')
      widget->text_offset[0].x = DATA_LEFT_OFFSET;
  }

  /** Draw widget base **/

  if (widget->data[DATA_D_OUTLINE])
  {
    temp_gc = Win->hue_widget[COLOR_CTRL_HIGH];
    Win->hue_widget[COLOR_CTRL_HIGH] = Win->hue_widget[COLOR_CTRL_SHADOW];
  }

  draw_generic_box (widget, BULGE_IN, BULGE_KEEP, ACTIVE_SHOW);

  if (widget->data[DATA_D_OUTLINE])
    Win->hue_widget[COLOR_CTRL_HIGH] = temp_gc;

  /** Set convenience variables **/

  color_font_0 = widget->gcs[GC_FONT_FIRST];
  color_font_1 = widget->gcs[GC_FONT_SECOND];
  font_0 = widget->fonts[FONT_FIRST];
  font_1 = widget->fonts[FONT_SECOND];

  /** If not drawing multiple lines **/

  if (widget->data[DATA_D_LINE_COUNT] == 1)
  {
    if (widget->states[1])
    {
      XFillRectangle (Win->display, widget->drawable,
		Win->hue_widget[widget->gcs[DATA_G_SELECT_BACK]],
		widget->position.x + widget->text_offset[0].x +
			widget->states[0] * Win->fonts[font_0]->dimension.x,
		widget->position.y,
		widget->states[1] * Win->fonts[font_0]->dimension.x,
		widget->size.y);

      strcpy (string, widget->label);
      string[widget->states[0]] = '\0';

      draw_string (widget->drawable,
		widget->position.x, widget->position.y,
		widget->text_offset[0].x, widget->text_offset[0].y,
		string, font_0,
		Win->hue_widget[color_font_0],
		FALSE);

      strcpy (string, &widget->label[widget->states[0]]);
      string[widget->states[1]] = '\0';

      draw_string (widget->drawable,
		widget->position.x, widget->position.y,
		widget->text_offset[0].x + widget->states[0] *
			Win->fonts[font_0]->dimension.x,
		widget->text_offset[0].y,
		string, font_0,
		Win->hue_widget[widget->gcs[DATA_G_SELECT_FORE]],
		FALSE);

      strcpy (string, &widget->label[widget->states[0] + widget->states[1]]);

      draw_string (widget->drawable,
		widget->position.x, widget->position.y,
		widget->text_offset[0].x +
			(widget->states[0] + widget->states[1]) *
			Win->fonts[font_0]->dimension.x,
		widget->text_offset[0].y,
		string, font_0,
		Win->hue_widget[color_font_0],
		FALSE);
    }
    else
      draw_string (widget->drawable,
		widget->position.x, widget->position.y,
		widget->text_offset[0].x, widget->text_offset[0].y,
		widget->label, font_0,
		Win->hue_widget[color_font_0],
		FALSE);
  }

  /** Else drawing multiple lines **/

  else
  {
    line_count = MIN (widget->state_count, widget->data[DATA_D_LINE_COUNT]);

    /** For each text line **/

    for (i=0; i<line_count; i++)
    {
      /** If selected, draw as highlighted, else draw normal **/

      if (widget->state+i >= widget->state_count)
        continue;
      else if (widget->state+i == widget->next_state)
      {
        XFillRectangle (Win->display, widget->drawable,
		Win->hue_widget[widget->gcs[DATA_G_SELECT_BACK]],
		widget->position.x,
		widget->position.y + widget->text_offset[0].y +
				widget->text_offset[1].y * (i+1) -
				Win->fonts[font_0]->dimension.y - 2,
		widget->size.x, Win->fonts[font_0]->dimension.y + 4);

        if (widget->data[DATA_D_X_OFFSET] <
			strlen(widget->superlabels[widget->state+i]))
          draw_string (widget->drawable,
		widget->position.x, widget->position.y,
		widget->text_offset[0].x,
		widget->text_offset[0].y + widget->text_offset[1].y * (i+1),
		widget->superlabels[widget->state+i] +
			widget->data[DATA_D_X_OFFSET],
		font_0,
		Win->hue_widget[widget->gcs[DATA_G_SELECT_FORE]],
		widget->size.x - 2*widget->text_offset[0].x);
      }
      else
      {
        if (widget->data[DATA_D_X_OFFSET] <
			strlen(widget->superlabels[widget->state+i]))
          draw_string (widget->drawable,
		widget->position.x, widget->position.y,
		widget->text_offset[0].x,
		widget->text_offset[0].y + widget->text_offset[1].y * (i+1),
		widget->superlabels[widget->state+i] +
			widget->data[DATA_D_X_OFFSET],
		font_0,
		Win->hue_widget[color_font_0],
		widget->size.x - 2*widget->text_offset[0].x);
      }
    }
  }

  /** Draw descriptive label, if there is one **/

  if (widget->locate != LABEL_NONE)
    draw_string (widget->drawable,
                        widget->position.x, widget->position.y,
                        widget->text_offset[1].x, widget->text_offset[1].y,
                        widget->string, font_1,
                        Win->hue_widget[color_font_1],
                        FALSE);
}
