#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include "pip_constant.h"
#include "pip_extern.h"
#include "pip_generic.h"

#include "pip_c_view.h"


/******************************************************************************
  view_function (widget, line, function)

  Run the type-specific routine for widget <widget> that is associated with
  command <function>, using supplemental information from string <line>.
******************************************************************************/

view_function (widget, line, function)
  Widget *widget;
  char *line;
  short function;
{
  switch (function)
  {
    case FUNCTION_HELP:
      view_help (Console->formats[CONTROL_VIEW]);
      break;

    case FUNCTION_FORMAT:
      view_format (Console->formats[CONTROL_VIEW]);
      break;

    case FUNCTION_LOAD:
      parse_format_defaults (Console->formats[CONTROL_VIEW], line);
      break;

    case FUNCTION_STORE:
      break;

    case FUNCTION_DUMP:
      generic_dump (widget);
      break;

    case FUNCTION_INITIALIZE:
      view_initialize (widget);
      break;

    case FUNCTION_PARSE:
      break;

    case FUNCTION_CLEAN:
      view_clean (widget);
      break;

    case FUNCTION_SETUP:
      view_setup (widget);
      break;

    case FUNCTION_EDIT:
      break;

    case FUNCTION_SELECT:
      break;

    case FUNCTION_MOTION:
      break;
 
    case FUNCTION_RELEASE:
      break;

    case FUNCTION_EXECUTE:
      view_execute (widget);
      break;

    case FUNCTION_DRAW:
      view_draw (widget);
      break;

    default:
      throw_warning ("Function unknown to view", NULL);
      break;
  }
}



/******************************************************************************
  view_help (format)

  Print a help message to stdout detailing the usage of any type-specific
  arguments for the widget <format>, including default values when possible.
******************************************************************************/

view_help (format)
  Widget *format;
{
  printf ("\n");
  printf ("VIEW\n");
  printf ("    TEXT   <string>                view label\n");
  printf ("    LOCATE <locate>                relative position of label\n");
  printf ("\n");
}



/******************************************************************************
  view_format (format)

  Set type-specific default for canonical widget <format>.
******************************************************************************/

view_format (format)
  Widget *format;
{
  format->status[IS_SELECTABLE] =	TRUE;
  format->status[IS_ACTIVATABLE] =	FALSE;
  format->status[IS_EDITABLE] =		EDITABLE_ACTION;
  format->status[IS_MOTION_ACTIVE] =	TRUE;
  format->status[IS_BUFFERABLE] =	BUFFER_NORMAL;

  format->size.x =			DEFAULT_VIEW_DIMENSION_X;
  format->size.y =			DEFAULT_VIEW_DIMENSION_Y;
  format->spacing.x =			DEFAULT_VIEW_SPACING_X;
  format->spacing.y =			DEFAULT_VIEW_SPACING_Y;

  format->thickness[THICKNESS_SHADOW] =	2;
  format->thickness[THICKNESS_ACTIVE] =	2;

  format->data[VIEW_D_DRAWABLE] =	DEFAULT_VIEW_DRAWABLE;

  format->gcs[GC_BODY] =		COLOR_CTRL_DARK;
  format->gcs[GC_BODY_INV] =		COLOR_CTRL_DARK_INV;
  format->gcs[GC_FONT_FIRST] =		COLOR_CTRL_TEXT_3;

  format->fonts[FONT_FIRST] =		FONT_LARGE;
  format->fonts[FONT_SECOND] =		FONT_LARGE;

  format->locate =			LABEL_SOUTH_EAST;

  format->format =			CONTROL_VIEW;
}



/******************************************************************************
  view_initialize (widget)

  Initialize widget <widget> with non-canonical information.
******************************************************************************/

view_initialize (widget)
  Widget *widget;
{
  widget->format = CONTROL_VIEW;
}



/******************************************************************************
  view_clean (widget)

  Clear up any parameter discrepancies in widget <widget>.
******************************************************************************/

view_clean (widget)
  Widget *widget;
{
  if (!widget->state_count)
    widget->state = -1;
}



/******************************************************************************
  view_setup (widget)

  Set up derived parameters (those not directly accessible to user) for
  widget <widget>.
******************************************************************************/

view_setup (widget)
  Widget *widget;
{

  if (widget->state < 0)
     widget->locate = LABEL_NONE;
  else
    locate_text (widget, widget->label, 0, FONT_FIRST, TRUE);

  widget->bound_offset.x = widget->thickness[THICKNESS_SHADOW] +
                widget->thickness[THICKNESS_ACTIVE];
  widget->bound_offset.y = widget->thickness[THICKNESS_SHADOW] +
                widget->thickness[THICKNESS_ACTIVE];
  widget->bound_size.x = 2*widget->bound_offset.x + widget->size.x;
  widget->bound_size.y = 2*widget->bound_offset.y + widget->size.y;
}



/******************************************************************************
  view_execute (widget)

  Execute the function of widget <widget>, as when mouse button is released
  or mouse motion occurs.
******************************************************************************/

view_execute (widget)
  Widget *widget;
{
  widget->flag = TRUE;
}



/******************************************************************************
  view_draw (widget)

  Draw the widget <widget>.
******************************************************************************/

view_draw (widget)
  Widget *widget;
{
  short color_font_0,
        font_0,
        shadow_thickness;

  color_font_0 = widget->gcs[GC_FONT_FIRST];
  font_0 = widget->fonts[FONT_FIRST];
  shadow_thickness = widget->thickness[THICKNESS_SHADOW];

  if (widget->data[VIEW_D_DRAWABLE])
    draw_generic_box (widget, BULGE_IN, BULGE_KEEP, ACTIVE_SHOW);
  else
    draw_border_short (widget->drawable, widget,
			shadow_thickness,
			Win->hue_widget[COLOR_CTRL_SHADOW],
			Win->hue_widget[COLOR_CTRL_HIGH]);
 
  if (widget->locate != LABEL_NONE)
    draw_string (widget->drawable,
		widget->position.x, widget->position.y,
		widget->text_offset[0].x, widget->text_offset[0].y,
		widget->label, font_0,
		Win->hue_widget[color_font_0], FALSE);
}
