#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include "pip_constant.h"
#include "pip_extern.h"



/******************************************************************************
  draw_all_panels ()

  Draw all currently open panels.
******************************************************************************/

draw_all_panels ()
{
  int i;

  for (i=0; i<Console->panel_count; i++)
  {
    if (Console->panels[i]->is_open)
      draw_panel (Console->panels[i], TRUE);
  }
}



/******************************************************************************
  draw_panel (panel, clear)

  Draw panel <panel> in correct widget order.  If <use_clear>, clear the
  window prior to drawing (outdated).
******************************************************************************/

draw_panel (panel, use_clear)
  Panel *panel;
  int use_clear;
{
  int i, j,
      temp,
      areas[MAX_CONTROLS],
      indices[MAX_CONTROLS];

  Widget *widget;

  if (!panel->is_open)
    return;

  if (panel->window == (Window)(0))
    return;

/**
  if (use_clear)
    XFillRectangle (Win->display, panel->window,
			Win->hue_widget[COLOR_CTRL_BACK], 
			0, 0, panel->size.x, panel->size.y);
**/

  /** Compute the areas of each widget **/

  for (i=0; i<panel->widget_count; i++)
  {
    areas[i] = (panel->widgets[i]->size.x * panel->widgets[i]->size.y);
    indices[i] = i;
  }

  /** Rank order widgets by area **/

  for (i=0; i<panel->widget_count; i++)
  {
    for (j=1; j<panel->widget_count; j++)
    {
      if (areas[j] > areas[j-1])
      {
        temp = areas[j];
        areas[j] = areas[j-1];
        areas[j-1] = temp;

        temp = indices[j];
        indices[j] = indices[j-1];
        indices[j-1] = temp;
      }
    }
  }

  /** Draw widgets in reverse-area ordering (largest to smallest) **/

  for (i=0; i<panel->widget_count; i++)
    draw_widget (panel->widgets[indices[i]]);

  XSync (Win->display, 0);
}



/******************************************************************************
  draw_widget (widget)

  Draw single widget <widget>, making sure to handle double buffering
  appropriately.
******************************************************************************/

draw_widget (widget)
  Widget *widget;
{
  unsigned char superformat;

  /** Make sure widget is enabled and panel open **/

  if (!widget->enable)
    return;

  if (!widget->panel->is_open)
    return;

  /** If widget should be double-buffered **/

  if (Master->use_buffer == BUFFER_WIDGET && widget->status[IS_BUFFERABLE])
  {
    /** Set buffer position **/

    widget->position.x = widget->bound_offset.x;
    widget->position.y = widget->bound_offset.y;

    /** Note the major hack to keep superformats from messing up and **/
    /** getting overdrawn on background pixmap before copying        **/

    superformat = widget->superformat;
    widget->superformat = CONTROL_X_NONE;

    /** Blank out the background if widget has transparent parts **/

    if (widget->status[IS_BUFFERABLE] == BUFFER_CLEAR && widget->background)
      XFillRectangle (Win->display, Console->pixmap,
		Win->hue_widget[widget->background->gcs[GC_BODY]],
		0, 0,
		widget->bound_size.x,
		widget->bound_size.y);

    /** Draw the widget **/

    widget_function (widget, NULL, FUNCTION_DRAW);

    /** Restore superformat **/

    widget->superformat = superformat;

    /** Copy widget to window **/

    XCopyArea (Win->display, Console->pixmap, widget->panel->window,
		Win->hue_widget[COLOR_CTRL_BACK], 
		0, 0, widget->bound_size.x, widget->bound_size.y,
		widget->real_position.x-widget->bound_offset.x,
		widget->real_position.y-widget->bound_offset.y);

    /** Restore widget position **/

    widget->position.x = widget->real_position.x;
    widget->position.y = widget->real_position.y;

    /** Handle drawing of superformat widget **/

    if (superformat != CONTROL_X_NONE)
      widget_function (widget->relatives[0], widget->name, FUNCTION_DRAW);
  }
  else if (Master->use_buffer == BUFFER_PANEL)
  {
    widget_function (widget, NULL, FUNCTION_DRAW);

    XCopyArea (Win->display, widget->panel->pixmap, widget->panel->window,
		Win->hue_widget[COLOR_CTRL_BACK], 
		widget->real_position.x-widget->bound_offset.x,
		widget->real_position.y-widget->bound_offset.y,
		widget->bound_size.x, widget->bound_size.y,
		widget->real_position.x-widget->bound_offset.x,
		widget->real_position.y-widget->bound_offset.y);
  }
  else
    widget_function (widget, NULL, FUNCTION_DRAW);
}



/******************************************************************************
  draw_overlay_widget (widget)

  Draw widget <widget>, along with any widgets overlaying it.  Used primarily
  for redrawing plates.  Doesn't handle hierarchical overlays.
******************************************************************************/

draw_overlay_widget (widget)
  Widget *widget;
{
  int i;
  Panel *panel;

  draw_widget (widget);

  panel = widget->panel;

  for (i=0; i<panel->widget_count; i++)
    if (panel->widgets[i]->background == widget)
      draw_widget (panel->widgets[i]);
}



/******************************************************************************
  draw_string (drawable, x_position, y_position, x_offset, y_offset,
				string, font_index, use_gc, x_limit)

  Draw text <string> on <drawable>, at appropriate position, with appropriate
  font and GC.
******************************************************************************/

draw_string (drawable, x_position, y_position, x_offset, y_offset,
				string, font_index, use_gc, x_limit)
  Window drawable;
  int x_position, y_position,
      x_offset, y_offset,
      x_limit,
      font_index;
  char string[];
  GC use_gc;
{
  int char_limit;
  XTextItem line_information[2];
  char *strdup();

  if (string == NULL)
    return;

  line_information[0].chars = strdup (string);
  line_information[0].nchars = strlen (string);

  if (x_limit)
  {
    char_limit = x_limit / Win->fonts[font_index]->dimension.x;
    if (char_limit < line_information[0].nchars)
      line_information[0].nchars = char_limit;
  }

  line_information[0].font = Win->fonts[font_index]->font->fid;
  line_information[0].delta = 0;

  XDrawText (Win->display, drawable, use_gc,
		x_position+x_offset, y_position+y_offset, line_information, 1);

  free (line_information[0].chars);
}



/******************************************************************************
  draw_generic_box (widget, use_bulge_out, swap_bulge, show_activity)

  Draw a generic shadowed box around widget <widget>, as specified by
  parameters within the widget structure.  If <use_bulge_out>, widget should
  appear raised.  If <swap_bulge> and widget selected, invert <use_bulge_out>.
  If <show_activity>, show select/active border around widget.
******************************************************************************/

draw_generic_box (widget, use_bulge_out, swap_bulge, show_activity)
  Widget *widget;
  short use_bulge_out,
        swap_bulge,
        show_activity;
{
  short color_surface,
        color_surface_inv,
        color_background,
        shadow_thickness,
        active_thickness;

  color_background = widget->gcs[GC_BACKGROUND];
  color_surface = widget->gcs[GC_BODY];
  if (widget->graphic_count)
    color_surface_inv = widget->gcs[GC_BODY];
  else
    color_surface_inv = widget->gcs[GC_BODY_INV];

  shadow_thickness = widget->thickness[THICKNESS_SHADOW];
  active_thickness = widget->thickness[THICKNESS_ACTIVE];

  if (widget->mode == MODE_SELECT)
    XFillRectangle (Win->display, widget->drawable,
                        Win->hue_widget[color_surface_inv],
                        widget->position.x, widget->position.y,
                        widget->size.x, widget->size.y);
  else
    XFillRectangle (Win->display, widget->drawable,
                        Win->hue_widget[color_surface],
                        widget->position.x, widget->position.y,
                        widget->size.x, widget->size.y);

  if (swap_bulge && widget->mode == MODE_SELECT)
    use_bulge_out = !use_bulge_out;
 
  if (use_bulge_out)
    draw_border_short (widget->drawable, widget,
                        shadow_thickness,
                        Win->hue_widget[COLOR_CTRL_HIGH],
                        Win->hue_widget[COLOR_CTRL_SHADOW]);
  else
    draw_border_short (widget->drawable, widget,
                        shadow_thickness,
                        Win->hue_widget[COLOR_CTRL_SHADOW],
                        Win->hue_widget[COLOR_CTRL_HIGH]);

  if (!show_activity)
    return;

  if (widget->mode == MODE_SELECT || widget->mode == MODE_ACTIVE)
    draw_border (widget->drawable,
                widget->position.x - shadow_thickness,
                widget->position.y - shadow_thickness,
                widget->size.x + 2*shadow_thickness,
                widget->size.y + 2*shadow_thickness, active_thickness,
                Win->hue_widget[COLOR_CTRL_ACTIVE],
                Win->hue_widget[COLOR_CTRL_ACTIVE]);
  else
    draw_border (widget->drawable,
                widget->position.x - shadow_thickness,
                widget->position.y - shadow_thickness,
                widget->size.x + 2*shadow_thickness,
                widget->size.y + 2*shadow_thickness, active_thickness,
                Win->hue_widget[color_background],
                Win->hue_widget[color_background]);
}



/******************************************************************************
  draw_border_short (drawable, widget, thickness, top_gc, bottom_gc)

  Shorthand method to draw boarder around widget <widget>, with a call to
  draw_border() using internal widget positioning and size.
******************************************************************************/

draw_border_short (drawable, widget, thickness, top_gc, bottom_gc)
  Window drawable;
  Widget *widget;
  int thickness;
  GC top_gc, bottom_gc;
{
  draw_border (drawable, widget->position.x, widget->position.y,
		widget->size.x, widget->size.y, thickness, top_gc, bottom_gc);
}



/******************************************************************************
  draw_border (drawable, xpos, ypos, x_size, y_size,
				thickness, top_gc, bottom_gc)

  Draw a raised or inset border of specified size, position, and colors.
******************************************************************************/

draw_border (drawable, xpos, ypos, x_size, y_size, thickness, top_gc, bottom_gc)
  Window drawable;
  int xpos, ypos,
      x_size, y_size,
      thickness;
  GC top_gc, bottom_gc;
{
  int i,
      xsrc, ysrc,
      xdest, ydest;

  xsrc = xpos - thickness;
  ysrc = ypos - thickness;

  xdest = xpos + x_size + thickness - 1;
  ydest = ypos + y_size + thickness - 1;

  for (i=0; i<thickness; i++)
  {
    XDrawLine (Win->display, drawable, top_gc, xsrc, ysrc+i, xdest-i, ysrc+i);
    XDrawLine (Win->display, drawable, top_gc, xsrc+i, ysrc, xsrc+i, ydest-i);

    XDrawLine (Win->display, drawable, bottom_gc,
			xsrc+i, ydest-i, xdest, ydest-i);
    XDrawLine (Win->display, drawable, bottom_gc,
			xdest-i, ydest-1, xdest-i, ysrc+i);
  }
}



/******************************************************************************
  draw_blank_region (drawable, widget, use_gc)

  Draw a blank region of GC <use_gc> covering widget <widget>.
******************************************************************************/

draw_blank_region (drawable, widget, use_gc)
  Window drawable;
  Widget *widget;
  GC use_gc;
{
  XFillRectangle (Win->display, drawable, use_gc,
			widget->position.x, widget->position.y,
			widget->size.x, widget->size.y);
}



/******************************************************************************
  get_color (triplet)

******************************************************************************/

get_color (triplet)
  unsigned char triplet[3];
{
  int i,
      red, green, blue; 

  unsigned long pixels[2];

  red =		triplet[0] << 8;
  green =	triplet[1] << 8;
  blue =	triplet[2] << 8;
 
  /** Search the existing colormap in widget colors **/
 
  for (i=0; i<Win->color_widget_limit; i++)
  {
    if (Win->color_widget[i].red == red &&
		Win->color_widget[i].green == green &&
		Win->color_widget[i].blue == blue)
      return (Win->color_widget[i].pixel);
  }

  /** Search the existing colormap in usable colors **/

/**
  for (i=0; i<Win->color_usable_limit; i++)
  {
    if (Win->color_usable[i].red == red &&
		Win->color_usable[i].green == green &&
		Win->color_usable[i].blue == blue)
      return (Win->color_usable[i].pixel);
  }
**/

  /** If there is space, allocate a new colormap entry **/

  if (Win->color_widget_limit < COLOR_CONTROL_MAX_LIMIT)
  {
    Win->color_widget[i].red =		red;
    Win->color_widget[i].green =	green;
    Win->color_widget[i].blue =		blue;
    Win->color_widget[i].flags =	DoRed | DoGreen | DoBlue;

    XAllocColorCells (Win->display, Win->colormap, FALSE, NULL, 0,
				pixels, 1);
    Win->color_widget[i].pixel =	pixels[0];
    XStoreColors (Win->display, Win->colormap, &(Win->color_widget[i]), 1);

    Win->color_widget_limit++;

    return (Win->color_widget[i].pixel);
  }
  else
    return (Win->color_widget[0].pixel);
}

