#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include "pip_constant.h"
#include "pip_extern.h"
#include "pip_generic.h"


/******************************************************************************
  int
  widget_function (widget, line, function)

  Handle generic widget events for <widget>, as specified by <function> and
  supplemented by the string <line>.  Return TRUE if the widget is not the
  panel, else return FALSE.
******************************************************************************/

int
widget_function (widget, line, function)
  Widget *widget;
  char *line;
  short function;
{
  short format;

  /** Set widget format type **/

  if (function == FUNCTION_FORMAT)
    format = CONTROL_ALL;
  else if (function == FUNCTION_INITIALIZE || function == FUNCTION_LOAD ||
			function == FUNCTION_HELP)
  {
    switch (line[0])
    {
#if USE_ARROW
      case 'A':
      case 'a':
        format = CONTROL_ARROW;
        break;
#endif

      case 'B':
      case 'b':
        switch (line[1])
        {
          case 'A':
          case 'a':
#if USE_BAR
            if (line[2] == 'R')
              format = CONTROL_BAR;
#endif
#if USE_BASE
            if (line[2] == 'S')
              format = CONTROL_BASE;
#endif
            break;

#if USE_BUTTON
          case 'U':
          case 'u':
            format = CONTROL_BUTTON;
            break;
#endif
        }
        break;

      case 'D':
      case 'd':
        switch (line[1])
        {
#if USE_DIAL
          case 'I':
          case 'i':
            format = CONTROL_DIAL;
            break;
#endif

#if USE_DATA
          case 'A':
          case 'a':
            format = CONTROL_DATA;
            break;
#endif
        }
        break;

      case 'L':
      case 'l':
        switch (line[1])
        {
#if USE_LABEL
          case 'A':
          case 'a':
            format = CONTROL_LABEL;
            break;
#endif

#if USE_LIST
          case 'I':
          case 'i':
            format = CONTROL_LIST;
            break;
#endif
        }
        break;

      case 'P':
      case 'p':
        switch (line[1])
        {
          case 'A':
          case 'a':
            return (FALSE);	/** it's the panel!! **/
            break;

#if USE_PLATE
          case 'L':
          case 'l':
            format = CONTROL_PLATE;
            break;
#endif
        }
        break;

      case 'S':
      case 's':
        switch (line[3])
        {
#if USE_SCROLL
          case 'O':
          case 'o':
            format = CONTROL_SCROLL;
            break;
#endif

#if USE_SLIDER
          case 'D':
          case 'd':
            format = CONTROL_SLIDER;
            break;
#endif

#if USE_SLIPPER
          case 'P':
          case 'p':
            format = CONTROL_SLIPPER;
            break;
#endif
        }
        break;

#if USE_DATA
      case 'T':		/** compatibility with old TEXT format **/
      case 't':
        format = CONTROL_DATA;
        break;
#endif

#if USE_VIEW
      case 'V':
      case 'v':
        format = CONTROL_VIEW;
        break;
#endif

      default:
        throw_error ("Unknown widget %s", line);
        break;
    }
    if (function == FUNCTION_INITIALIZE)
      widget->format = format;
  }
  else
    format = widget->format;

  /** Perform format-specific action **/
 
  switch (format)
  {
#if USE_ARROW
    case CONTROL_ARROW:
      arrow_function (widget, line, function);
      break;
#endif

#if USE_BAR
    case CONTROL_BAR:
      bar_function (widget, line, function);
      break;
#endif

#if USE_BASE
    case CONTROL_BASE:
      base_function (widget, line, function);
      break;
#endif

#if USE_BUTTON
    case CONTROL_BUTTON:
      button_function (widget, line, function);
      break;
#endif

#if USE_DATA
    case CONTROL_DATA:
      data_function (widget, line, function);
      break;
#endif

#if USE_DIAL
    case CONTROL_DIAL:
      dial_function (widget, line, function);
      break;
#endif

#if USE_LABEL
    case CONTROL_LABEL:
      label_function (widget, line, function);
      break;
#endif

#if USE_LIST
    case CONTROL_LIST:
      list_function (widget, line, function);
      break;
#endif

#if USE_PLATE
    case CONTROL_PLATE:
      plate_function (widget, line, function);
      break;
#endif

#if USE_SCROLL
    case CONTROL_SCROLL:
      scroll_function (widget, line, function);
      break;
#endif

#if USE_SLIDER
    case CONTROL_SLIDER:
      slider_function (widget, line, function);
      break;
#endif

#if USE_SLIPPER
    case CONTROL_SLIPPER:
      slipper_function (widget, line, function);
      break;
#endif

#if USE_VIEW
    case CONTROL_VIEW:
      view_function (widget, line, function);
      break;
#endif

    case CONTROL_ALL:
#if USE_ARROW
      arrow_function (widget, line, function);
#endif
#if USE_BAR
      bar_function (widget, line, function);
#endif
#if USE_BASE
      base_function (widget, line, function);
#endif
#if USE_BUTTON
      button_function (widget, line, function);
#endif
#if USE_DATA
      data_function (widget, line, function);
#endif
#if USE_DIAL
      dial_function (widget, line, function);
#endif
#if USE_LABEL
      label_function (widget, line, function);
#endif
#if USE_LIST
      list_function (widget, line, function);
#endif
#if USE_PLATE
      plate_function (widget, line, function);
#endif
#if USE_SCROLL
      scroll_function (widget, line, function);
#endif
#if USE_SLIDER
      slider_function (widget, line, function);
#endif
#if USE_SLIPPER
      slipper_function (widget, line, function);
#endif
#if USE_VIEW
      view_function (widget, line, function);
#endif
      return;
      break;
  }

  /** If executing and widget activatible, redraw widget **/

  if (function == FUNCTION_EXECUTE)
  {
    if (widget->status[IS_ACTIVATABLE])
    {
      draw_widget (widget);
      XSync (Win->display, 0);
    }
  }

  /** If widget exists and has a superformat **/

  if (widget != NULL && widget->superformat != CONTROL_NONE)
  {
    /** Perform certain function of superformat relative **/

    switch (function)
    {
      case FUNCTION_EDIT:
      case FUNCTION_EXECUTE:
      case FUNCTION_DRAW:
        widget_function (widget->relatives[0], widget->name, function);
        break;
    }
  }

  return (TRUE);
}



/******************************************************************************
  generic_dump (widget)

  Dump generic widget info (name, position, and size) for <widget>.
******************************************************************************/

generic_dump (widget)
  Widget *widget;
{
  printf ("%s\n", widget->name);
  printf ("  x: %3d   y: %3d      w: %3d  h: %3d\n",
		widget->position.x, widget->position.y,
		widget->size.x, widget->size.y);
}
