#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include "pip_constant.h"
#include "pip_extern.h"


/******************************************************************************
  initialize_parameters ()

  Initiliaze various global parameters.
******************************************************************************/

initialize_parameters ()
{
  initialize_window_parameters ();

  initialize_console_parameters ();

  initialize_global_parameters ();
}



/******************************************************************************
  initialize_window_parameters ()

  Initialize the global <Win> parameters.
******************************************************************************/

initialize_window_parameters ()
{
  Win = (struct xwindow_info *)(malloc(sizeof(struct xwindow_info)));

  Win->hint.width =			DEFAULT_WIDTH;
  Win->hint.height =			DEFAULT_HEIGHT;
  Win->hint.x =				DEFAULT_XPOS;
  Win->hint.y =				DEFAULT_YPOS;

  Win->font_limit =			FONT_DEFAULT_LIMIT;

  Win->color_orable_limit =		COLOR_ORABLE_DEFAULT_LIMIT;
  Win->color_usable_limit =		COLOR_USABLE_DEFAULT_LIMIT;
  Win->color_widget_limit =		COLOR_CONTROL_DEFAULT_LIMIT;
  Win->use_dedicated_colormap =		FALSE;

  strcpy (Win->display_name,		"");
  strcpy (Win->window_title,		"EXTENDED PIP");

  initialize_colors ();
  initialize_fonts ();
  initialize_spectrum ();
  initialize_typeset ();
}



/******************************************************************************
  initialize_console_parameters ()

  Initialize the global <Console> parameters.
******************************************************************************/

initialize_console_parameters ()
{
  int i;

  Console = (struct console_info *)(malloc(sizeof(struct console_info)));

  strcpy (Console->panel_files[0], "xpip.if");
  Console->panel_files[1][0] = '\0';

  Console->base_panel = Console->panels[0];
  Console->panel = Console->base_panel;
  Console->next_panel = NULL;

  Console->dummy_widget = create_widget();

  for (i=0; i<MAX_FORMATS; i++)
  {
    Console->formats[i] = create_widget();
    Console->back_formats[i] = create_widget();

    initialize_generic_format (Console->formats[i]);
  }
  
  widget_function (NULL, NULL, FUNCTION_FORMAT);
}



/******************************************************************************
  initialize_global_parameters ()

  Initialize the global <Master> parameters.
******************************************************************************/

initialize_global_parameters ()
{
  Master = (struct master_info *)(malloc(sizeof(struct master_info)));

  strcpy (Master->widget_setup_file,	DEFAULT_SETUP_FILENAME);
  strcpy (Master->bitmap_directory,	DEFAULT_BITMAP_DIRECTORY);
  strcpy (Master->help_directory,	DEFAULT_HELP_DIRECTORY);

  Master->allow_drag =			TRUE;
  Master->last_value =			0.0;

  Master->use_offset_activate =		TRUE;
  Master->use_buffer =			BUFFER_NONE;
  Master->use_motion_highlights =	FALSE;
  Master->use_v_2_1_definitions =	FALSE;

  Master->time_delay =			BASE_DELAY;

}



/******************************************************************************
  initialize_panel (panel)

  Initialize <panel> with some sensible default values.
******************************************************************************/

initialize_panel (panel)
  Panel *panel;
{
  panel->widget_count =			0;
  panel->size.x =			100;
  panel->size.y =			100;
  panel->position.x =			0;
  panel->position.y =			0;
  panel->window =			(Window)(0);
  panel->pixmap =			(Pixmap)(0);
  panel->is_open =			FALSE;
  panel->is_active =			FALSE;
  panel->is_drawn =			FALSE;
  strcpy (panel->name,			"Panel");
}



/******************************************************************************
  initialize_generic_format (format)

  Initialize canonical widget <format> with non-type-specific defaults.  The
  type-specific defaults are set with widget_function(FUNCTION_FORMAT).
******************************************************************************/

initialize_generic_format (format)
  Widget *format;
{
  int i;

  format->format =			CONTROL_X_NONE;
  format->subformat =			CONTROL_X_NONE;
  format->superformat =			CONTROL_X_NONE;
  format->mode =			MODE_NORMAL;
  format->flag =			FALSE;
  format->enable = 			TRUE;
  format->reserve = 			EDIT_BEGIN;
  format->user =			TRUE;
  format->locate =			LABEL_CENTER;
  format->status[IS_SELECTABLE] =	TRUE;
  format->status[IS_ACTIVATABLE] =	TRUE;
  format->status[IS_EDITABLE] =		EDITABLE_NEVER;
  format->status[IS_MOTION_ACTIVE] =	FALSE;
  format->status[IS_TIMER_ACTIVE] =	FALSE;
  format->status[IS_BUFFERABLE] =	TRUE;
  format->status[IS_COMPOSITE] =	FALSE;
  format->status[IS_EVENT] =		EVENT_NOTHING;

  format->size.x =			DEFAULT_DIMENSION_X;
  format->size.y =			DEFAULT_DIMENSION_Y;
  format->position.x =			0;
  format->position.y =			0;
  format->real_position.x =		0;
  format->real_position.y =		0;
  format->spacing.x =			DEFAULT_SPACING_X;
  format->spacing.y =			DEFAULT_SPACING_Y;
  format->delta.x =			DEFAULT_DELTA_X;
  format->delta.y =			DEFAULT_DELTA_Y;
  format->cursor.x =			0;
  format->cursor.y =			0;
  format->locate_offset.x =		10;
  format->locate_offset.y =		10;
  for (i=0; i<MAX_STATES; i++)
  {
    format->text_offset[i].x =		0;
    format->text_offset[i].y =		0;
  }
  for (i=0; i<MAX_POINTS; i++)
  {
    format->points[i].x =		0;
    format->points[i].y =		0;
  }
  format->bound_offset.x =		0;
  format->bound_offset.y =		0;
  format->bound_size.x =		DEFAULT_DIMENSION_X;
  format->bound_size.y =		DEFAULT_DIMENSION_Y;

  format->thickness[THICKNESS_SHADOW] = 3;
  format->thickness[THICKNESS_ACTIVE] = 2;
  format->thickness[THICKNESS_SELECT] = 2;

  format->label =			NULL;
  format->string =			NULL;

  format->fonts[FONT_FIRST] =		FONT_MEDIUM;
  format->fonts[FONT_SECOND] =		FONT_MEDIUM;
  format->gcs[GC_BODY] =		COLOR_CTRL_SURFACE;
  format->gcs[GC_BODY_INV] =		COLOR_CTRL_SURFACE_INV;
  format->gcs[GC_FONT_FIRST] =		COLOR_CTRL_TEXT_1;
  format->gcs[GC_FONT_SECOND] =		COLOR_CTRL_TEXT_2;
  format->gcs[4] =			COLOR_CTRL_TEXT_3;

  format->state_count =			0;
  format->state = 			0;
  format->next_state = 			0;
  for (i=0; i<MAX_STATES; i++)
    format->states[i] =			0;

  format->value_count = 		0;
  format->value = 			0.0;
  format->next_value =			0.0;
  for (i=0; i<MAX_VALUES; i++)
    format->values[i] =			0.0;
  format->minimum =			0.0;
  format->maximum =			1.0;

  for (i=0; i<MAX_DATA; i++)
  {
    format->data[i] =			0;
    format->data_float[i] =		0.0;
  }

  format->graphic_count =		0;
}



/******************************************************************************
  initialize_pre_processes ()

  Initialize window paramters prior to setup variations.
******************************************************************************/

initialize_pre_processes ()
{
  load_widget_setup ();

  push_formats ();

  load_panel_info ();

  if (Console->panels[0]->size.x)
    Win->hint.width = Console->panels[0]->size.x;
  if (Console->panels[0]->size.y)
    Win->hint.height = Console->panels[0]->size.y;
  if (Console->panels[0]->position.x)
    Win->hint.x = Console->panels[0]->position.x;
  if (Console->panels[0]->position.y)
    Win->hint.y = Console->panels[0]->position.y;
  strcpy (Win->window_title, Console->panels[0]->name); 

  Console->panels[0]->size.x = Win->hint.width;
  Console->panels[0]->size.y = Win->hint.height;
  Console->panels[0]->position.x = Win->hint.x;
  Console->panels[0]->position.y = Win->hint.y;
}



/******************************************************************************
  initialize_post_processes ()

  Revise panel and widget parameters based on mutual specifications.  Most
  importantly, handle widget-relative placement.
******************************************************************************/

initialize_post_processes ()
{
  int i, j;

  /** For each panel **/

  for (i=0; i<Console->panel_count; i++)
  {
    /** If panel has no widgets, go to next panel **/

    if (Console->panels[i]->widgets == NULL)
      continue;

    /** For each widget, run type-specific setup **/

    for (j=0; j<Console->panels[i]->widget_count; j++)
      widget_function (Console->panels[i]->widgets[j], NULL, FUNCTION_SETUP);

    /** Set locations and sizes of each widget **/

    Console->panel = Console->panels[i];
    calculate_locations (Console->panel);

    /** For each widget, determine background GC **/

    for (j=0; j<Console->panel->widget_count; j++)
      init_background (Console->panel->widgets[j]);

    /** For each widget, set real (absolute) positions **/

    for (j=0; j<Console->panel->widget_count; j++)
    {
      Console->panel->widgets[j]->real_position.x =
			Console->panel->widgets[j]->position.x;
      Console->panel->widgets[j]->real_position.y =
			Console->panel->widgets[j]->position.y;
    }
  }

  Console->panel = Console->base_panel;

  if (Master->use_buffer != BUFFER_NONE)
    initialize_buffer ();

  for (i=0; i<Console->panel_count; i++)
    init_widgets_window (Console->panels[i]);
}



/******************************************************************************
  init_widgets_window (panel)

  Initialize drawable surfaces of each widget in <panel>.
******************************************************************************/

init_widgets_window (panel)
  Panel *panel;
{
  int i;
  for (i=0; i<panel->widget_count; i++)
  {
    if (Master->use_buffer == BUFFER_WIDGET &&
			panel->widgets[i]->status[IS_BUFFERABLE])
      panel->widgets[i]->drawable = Console->pixmap;
    else if (Master->use_buffer == BUFFER_PANEL)
      panel->widgets[i]->drawable = panel->pixmap;
    else
      panel->widgets[i]->drawable = panel->window;
  }
}



/******************************************************************************
  init_background (widget)

  Determine the background GC of widget <widget>.
******************************************************************************/

init_background (widget)
  Widget *widget;
{
  widget->background = strong_widget_search (widget->panel,
			widget->position.x - 1, widget->position.y);

  if (widget->background == NULL)
    widget->gcs[GC_BACKGROUND] = COLOR_CTRL_BACK;
  else
    widget->gcs[GC_BACKGROUND] = widget->background->gcs[GC_BODY];
}



/******************************************************************************
  initialize_buffer ()

  Initialize buffer for double buffering.
******************************************************************************/

initialize_buffer ()
{
  int i, j;

  XPoint max; 

  Panel *panel;

  if (Master->use_buffer == BUFFER_WIDGET)
  {
    /** Determine maximum widget dimensions **/

    max.x = 0;
    max.y = 0;

    for (i=0; i<Console->panel_count; i++)
      for (j=0; j<Console->panels[i]->widget_count; j++)
      {
        if (Console->panels[i]->widgets[j]->size.x > max.x)
          max.x = Console->panels[i]->widgets[j]->size.x;

        if (Console->panels[i]->widgets[j]->size.y > max.y)
          max.y = Console->panels[i]->widgets[j]->size.y;
      }

    /** ALTER: what the heck is this? **/

    max.x += 8*3;
    max.y += 8*3;

    /** Create pixmap for double buffering **/

    Console->pixmap =
	XCreatePixmap (Win->display, Win->window, max.x, max.y, Win->depth);

    for (i=0; i<Console->panel_count; i++)
      Console->panels[i]->pixmap = Console->pixmap;
  }
  else if (Master->use_buffer == BUFFER_PANEL)
  {
    for (i=0; i<Console->panel_count; i++)
    {
      panel = Console->panels[i];
      panel->pixmap = XCreatePixmap (Win->display, Win->window, 
		panel->size.x, panel->size.y, Win->depth);
    }
  }
}



/******************************************************************************
  initialize_colors ()

  Initialize default colors in <Win>.
******************************************************************************/

initialize_colors ()
{
  int i;

  for (i=0; i<COLOR_ORABLE_DEFAULT_LIMIT; i++)
  {
    Win->color_orable[i].red =   ColorOrable[i][0]<<8;
    Win->color_orable[i].green = ColorOrable[i][1]<<8;
    Win->color_orable[i].blue =  ColorOrable[i][2]<<8;
  }

  for (i=0; i<COLOR_CONTROL_DEFAULT_LIMIT; i++)
  {
    Win->color_widget[i].red =   ColorControl[i][0]<<8;
    Win->color_widget[i].green = ColorControl[i][1]<<8;
    Win->color_widget[i].blue =  ColorControl[i][2]<<8;
  }

  for (i=0; i<COLOR_USABLE_DEFAULT_LIMIT; i++)
  {
    Win->color_usable[i].red =   ColorUsable[i][0]<<8;
    Win->color_usable[i].green = ColorUsable[i][1]<<8;
    Win->color_usable[i].blue =  ColorUsable[i][2]<<8;
  }
}



/******************************************************************************
  initialize_spectrum ()

  Initialize default spectrum (names and definitions).
******************************************************************************/

initialize_spectrum ()
{
  int i;

  Win->spectrum_count = COLOR_CONTROL_DEFAULT_LIMIT;

  for (i=0; i<Win->spectrum_count; i++)
  {
    strcpy (Win->spectrum_names[i], Spectrum[i]);
    Win->spectrum_indices[i] = i;
    Win->spectrum_defs[i][0] = -1;
    Win->spectrum_defs[i][1] = -1;
    Win->spectrum_defs[i][2] = -1;
  }
}


/******************************************************************************
  initialize_typeset ()

  Initialize default typeset (font names and definitions).
******************************************************************************/

initialize_typeset ()
{
  int i;

  Win->typeset_count = FONT_DEFAULT_LIMIT;

  for (i=0; i<Win->typeset_count; i++)
  {
    strcpy (Win->typeset_names[i], Typeset[i]);
    Win->typeset_indices[i] = i;
  }
}



/******************************************************************************
  initialize_fonts ()

  Initialize default fonts in <Win>.
******************************************************************************/

initialize_fonts ()
{
  int i;

  for (i=0; i<FONT_MAX_LIMIT; i++)
    Win->fonts[i] = (struct font_info *)(malloc(sizeof(struct font_info)));

  Win->fonts[0]->fontname = strdup (DEFAULT_FONT_0);
  Win->fonts[1]->fontname = strdup (DEFAULT_FONT_1);
  Win->fonts[2]->fontname = strdup (DEFAULT_FONT_2);
  Win->fonts[3]->fontname = strdup (DEFAULT_FONT_3);
  Win->fonts[4]->fontname = strdup (DEFAULT_FONT_4);
}


/******************************************************************************
  mini_parse (line)

  Manipulate the arguments of string <line> such that they mimic normal
  command line arguments, then parse with normal parser.
******************************************************************************/

mini_parse (line)
  char *line;
{
  int i,
      argc;
  char *argv[3],
       temp_line[100],
       unos_line[100],
       dos_line[100],
       *strdup();

  /** Get the argument and parameters **/

  copy_first (temp_line, line);
  strcpy (dos_line, line);
  strip_first (dos_line);

  /** Add the appropriate argument prefix **/

  if (temp_line[0] != '-')
  {
    if (temp_line[0] != 'p' || temp_line[1] != 'i')
      sprintf (unos_line, "-pip_%s", temp_line);
    else
      sprintf (unos_line, "-%s", temp_line);
  }
  else
    sprintf (unos_line, "%s", temp_line);

  /** Uncapitalize argument **/

  for (i=0; i<strlen(unos_line); i++)
    if (unos_line[i] >= 'A' && unos_line[i] <= 'Z')
      unos_line[i] = unos_line[i] - 'A' + 'a';

  /** Create argc/argv structures **/

  argc = 2;
  argv[0] = NULL;
  argv[1] = strdup (unos_line);

  if (dos_line[0] != '\0')
  {
    argc = 3;
    argv[2] = strdup (dos_line);
  }

  /** Parse fake command line **/

  parse (&argc, argv);

  /** Free argv space **/

/**
  free (argv[1]);
  if (dos_line[0] != '\0')
    free (argv[2]);
**/
}



/******************************************************************************
  parse (argc, argv)

  Parse the command line.
******************************************************************************/

parse (argc, argv)
  int *argc;
  char **argv;
{
  int i,
      index,
       default_setup;

  static int first = TRUE;

  default_setup = FALSE;

  if (index = is_partial_arg("-help_", *argc, argv))
  {
    widget_function (NULL, &(argv[index][6]), FUNCTION_HELP);
    exit (0);
  }

  if (index = is_partial_arg("-help", *argc, argv))
  {
    print_help_message ();
    return;
  }

  if (index = is_partial_arg("-pip_x", *argc, argv))
  {
    Win->hint.width = atoi (argv[index+1]);
    erase_argv (argc, argv, index, 2);
  }

  if (index = is_partial_arg("-pip_y", *argc, argv))
  {
    Win->hint.height = atoi (argv[index+1]);
    erase_argv (argc, argv, index, 2);
  }

  if (index = is_partial_arg("-pip_s", *argc, argv))
  {
    Win->hint.width = atoi (argv[index+1]);
    Win->hint.height = atoi (argv[index+1]);
    erase_argv (argc, argv, index, 2);
  }

  if (index = is_partial_arg("-pip_px", *argc, argv))
  {
    Win->hint.x = atoi (argv[index+1]);
    erase_argv (argc, argv, index, 2);
  }

  if (index = is_partial_arg("-pip_py", *argc, argv))
  {
    Win->hint.y = atoi (argv[index+1]);
    erase_argv (argc, argv, index, 2);
  }

  if (index = is_partial_arg("-pip_de", *argc, argv))
  {
    Master->time_delay = atoi (argv[index+1]);
    erase_argv (argc, argv, index, 2);
  }

  if (index = is_partial_arg("-pip_wid", *argc, argv))
  {
    Master->use_buffer = BUFFER_WIDGET;
    erase_argv (argc, argv, index, 1);
  }

  if (index = is_partial_arg("-pip_pan", *argc, argv))
  {
    Master->use_buffer = BUFFER_PANEL;
    erase_argv (argc, argv, index, 1);
  }

  if (index = is_partial_arg("-pip_dou", *argc, argv))
  {
    Master->use_buffer = BUFFER_PANEL;
    erase_argv (argc, argv, index, 1);
  }

  if (index = is_partial_arg("-pip_mo", *argc, argv))
  {
    Master->use_motion_highlights = TRUE;
    erase_argv (argc, argv, index, 1);
  }

  if (index = is_partial_arg("-pip_col", *argc, argv))
  {
    Win->use_dedicated_colormap = TRUE;
    erase_argv (argc, argv, index, 1);
  }

  if (index = is_partial_arg("-pip_ons", *argc, argv))
  {
    Master->use_offset_activate = FALSE;
    erase_argv (argc, argv, index, 1);
  }

  if (index = is_partial_arg("-pip_v_2_1", *argc, argv))
  {
    Master->use_v_2_1_definitions = TRUE;
    erase_argv (argc, argv, index, 1);
  }

  if (index = is_partial_arg("-pip_con", *argc, argv))
  {
    strcpy (Master->widget_setup_file, argv[index+1]);
    erase_argv (argc, argv, index, 2);
  }

  if (first)
    first = FALSE;
}



/******************************************************************************
  finish_parse (argc, argv)

  If there are any commands left in the argc/argv composite, flag as errors.
******************************************************************************/

finish_parse (argc, argv)
  int argc;
  char *argv[];
{
  int i;

  if (argc != 1)
  {
    printf ("ILLEGAL ARGUMENT:   ");
    for (i=1; i<argc; i++)
      printf ("%s ", argv[i]);
    printf ("\n");
    exit (0);
  }
}



/******************************************************************************
  is_partial_arg (str, argc, argv)

  Check to see if argument given by <str> is in the current command line (as
  defined by <argc> and <argv>).  Return the command line index if the
  argument is found, else return 0.
******************************************************************************/

is_partial_arg (str, argc, argv)
  char str[],
       *argv[];
  int argc;
{
  int i;

  for (i=1; i<argc; i++)
    if (strncmp (argv[i],str,strlen(str))==0)
      return (i);
  return(FALSE);
}



/******************************************************************************
  erase_argv (argc, argv, index, count)

  Erase <count> arguments, starting with the <index>th argument, from the
  current command line (as defined by <argc> and <argv>).
******************************************************************************/

erase_argv (argc, argv, index, count)
  int *argc;
  char **argv;
  int index,
      count;
{
  int i;

  for (i=index; i<*argc; i++)
    argv[i] = argv[i+count];
  *argc -= count;
}


/******************************************************************************
  print_help_message ()

  Display information help message explaining the usage of xpip arguments.
******************************************************************************/

print_help_message ()
{
  printf ("\n");
  printf ("usage: xpip [-options ...]\n");
  printf ("\n");
  printf ("  -pip_x           <int>       width of pip window\n");
  printf ("  -pip_y           <int>       height of pip window\n");
  printf ("  -pip_s           <int>       width=height of pip window\n");
  printf ("  -pip_px          <int>       x position of pip window\n");
  printf ("  -pip_py          <int>       y position of pip window\n");
  printf ("  -pip_delay       <int>       delay between auto update\n");
  printf ("  -pip_control     <file>      use <file> instead of xpip.cf\n");
  printf ("  -pip_colormap                use dedicated colormap\n");
  printf ("  -pip_onset                   activate widgets by button onset\n");
  printf ("  -pip_double                  use per-widget double buffering\n");
  printf ("  -pip_widget                  use per-widget double buffering\n");
  printf ("  -pip_panel                   use per-panel double buffering\n");
  printf ("  -pip_motion                  highlight buttons from mouse motion\n");
  printf ("  -pip_v_2_1                   use version 2.1 if file definitions\n");
  printf ("\n");
}
