#include "pip_global.h"

#include <sys/types.h>
#include <sys/time.h>

#ifndef _SYS_SELECT_H
#ifndef FD_ZERO
#include <sys/select.h>
#endif
#endif


/******************************************************************************
  xpip

  The X Portable Interface Package is designed for the quick development and
  transfer of GUIs for various purposes.
******************************************************************************/

main (argc, argv)
  int argc;
  char *argv[];
{
  initialize_parameters ();

  parse (&argc, argv);

  initialize_pre_processes ();

  startup_routine (&argc, argv);

  finish_parse (argc, argv);

  open_base_window ();

  initialize_post_processes ();

  open_all_windows ();

  run_widget_loop ();
}



/******************************************************************************
  run_widget_loop ()

  Initialize panel situation and loop continuously through an event loop,
  widgetling events as appropriate.
******************************************************************************/

run_widget_loop ()
{
  int i,
      done,
      use_execute,
      textcount;

  char text[10];

  KeySym key;

  unsigned long new_time,
                old_time,
                target_time;

  Panel *event_panel;

  Widget *widget,
         *ptr;

  struct timeval old_tv,
                 timeout_tv;
  struct timezone old_tz;
  int number_fds,
      mask_fds,
      found_fds;
  fd_set set_fds,
         read_fds;

  /** Set some basic widget parameters **/

  Win->update_event = FALSE;

  Console->panel = Console->base_panel;
  Console->event_panel = Console->panel;
  Console->next_panel = NULL;

  Console->active_widget = NULL;
  Console->select_widget = NULL;

  /** Anyone know what this loop is doing? **/
  /** ALTER: Huh? **/

  done = FALSE;
  XFlush (Win->display);
  while (!done)
  {
    XNextEvent (Win->display, &(Win->event));
    switch (Win->event.type)
    {
      case Expose:
        if (Win->event.xexpose.count == 0)
          done = TRUE;
       break;

      default:
       break;
    }
  }
  XFlush (Win->display);
  XSync (Win->display, 0);


  for (i=0; i<Console->panel_count; i++)
  {
    if (Console->panels[i]->is_open)
    {
      set_active_panel (Console->panels[i]);

      /** Draw the currently active panel **/

      draw_panel (Console->panels[i], TRUE);

      /** Make sure XPIP knows panel has been drawn **/

      Console->panels[i]->is_drawn = TRUE;

      /** Init the current panel routine **/

      init_routine (Console->panels[i]);

      /** Execute the current panel routine **/

      execute_routine (Console->panels[i], Win->event);

      restore_panel ();
    }
  }

  /** Set up the select connection **/

  number_fds = ConnectionNumber(Win->display);
  mask_fds = 1<<ConnectionNumber(Win->display);
  FD_ZERO (&set_fds);
  FD_SET (ConnectionNumber(Win->display), &set_fds);

  /** We are using a kludgy version of the timer which has a delay of **/
  /** Master->time_delay/10000 seconds AFTER the last event.  This is **/
  /** decent for ButtonDown type of events, but is unacceptable for   **/
  /** accurate timing for animation and such.  See the end of this    **/
  /** file for a more accurate timer.				      **/	

  while (TRUE)
  {
    /** Set the event type information **/

    if (Win->update_event)
      Win->update_event = FALSE;
    Win->event.type = NoExpose;
    use_execute = TRUE;

    /** Only use select when there are no events in queue **/

    if (XEventsQueued (Win->display, QueuedAfterReading) <= 0)
    {
      gettimeofday (&old_tv, &old_tz);
      old_time = (old_tv.tv_sec%10000)*1000000 + old_tv.tv_usec;
      target_time = old_time + (Master->time_delay*100) +
					 (Master->time_delay%10000)*100;
      timeout_tv.tv_sec = (target_time - old_time)/1000000;
      timeout_tv.tv_usec = (target_time - old_time)%1000000;
      read_fds = set_fds;

      if (XEventsQueued (Win->display, QueuedAfterReading) <= 0)
        found_fds = select (number_fds+1, &read_fds, NULL, NULL, &timeout_tv);
      else
        found_fds = 1;
    }
    else
      found_fds = 1;

    /** Empty the event queue **/

    if (XEventsQueued (Win->display, QueuedAfterReading) > 0)
      XNextEvent (Win->display, &(Win->event));
    else if (!found_fds)
    {
      Win->update_event = TRUE;
      Win->event.type = KeymapNotify;
    }

    widget = NULL;

    /** Set the panel in which the event occurred **/

    event_panel = NULL;
    if (!Win->update_event)
    {
      for (i=0; i<Console->panel_count; i++)
      {
        if (Console->panels[i]->window == Win->event.xexpose.window)
        {
          event_panel = Console->panels[i];
          break;
        }
      }
    }
    else
    {
      if (Console->select_widget)
        event_panel = Console->select_widget->panel;
      else if (Console->active_widget)
        event_panel = Console->active_widget->panel;

      if (event_panel)
        Win->event.type = PropertyNotify;
      else
        Win->event.type = KeymapNotify;
    }

    if (event_panel == NULL)
      event_panel = Console->panels[0];

    Console->event_panel = event_panel;
    Console->panel = event_panel;

    /** Determine the event type and perform appropriate action **/

    switch (Win->event.type)
    {
      /** Some type of copy event **/

      case GraphicsExpose:

        use_execute = FALSE;
        break;

      /** A non-event **/

      case NoExpose:

        use_execute = FALSE;
        break;

      /** A non-event **/

      case KeymapNotify:

/**
        use_execute = FALSE;
**/
        break;

      /** Something has happened to a window **/

      case PropertyNotify:

        widget = handle_widget_search (event_panel);
        widget = handle_button_motion (event_panel, widget, EVENT_TIMER);
        if (widget == NULL)
          use_execute = FALSE;
        break;

      /** A window has changed appearance **/

      case ConfigureNotify:

        if (Win->event.xconfigure.width != event_panel->size.x ||
		Win->event.xconfigure.height != event_panel->size.y)
        {
          event_panel->size.x = Win->event.xconfigure.width;
          event_panel->size.y = Win->event.xconfigure.height;
          draw_panel (event_panel, TRUE);
        }
        else
          Win->event.type = NoExpose;
        break;

      /** A window has been exposed **/

      case Expose:

        if (Win->event.xexpose.count == 0)
        {
          if (Master->use_buffer == BUFFER_PANEL && event_panel->is_drawn)
          {
            XCopyArea (Win->display,
			event_panel->pixmap, event_panel->window,
                	Win->hue_widget[COLOR_CTRL_BACK],
                	0, 0,
			event_panel->size.x, event_panel->size.y,
                	0, 0);
            XSync (Win->display, 0);
          }
          else
          {
            draw_panel (event_panel, TRUE);
            event_panel->is_drawn = TRUE;
          }
        }
        break;

      /** A mouse button has been pressed **/

      case ButtonPress:

        widget = handle_widget_search (event_panel);
        widget = handle_button_press (event_panel, widget);
        break;

      /** A mouse button has been released **/

      case ButtonRelease:

        widget = handle_widget_search (event_panel);
        widget = handle_button_release (event_panel, widget);
        break;

      /** The mouse has moved **/

      case MotionNotify:

        widget = handle_widget_search (event_panel);
        widget = handle_button_motion (event_panel, widget, EVENT_MOTION);
        break;

      /** A key has been pressed **/

      case KeyPress:

        widget = handle_widget_search (event_panel);
        widget = handle_keypress (event_panel, widget);

        /** If cursor was not in key-activatible widget **/

        if (widget == NULL)
        {
          textcount = XLookupString((XKeyEvent *)(&(Win->event)),
					text, 10, &key, NULL);
          if (textcount != 0)
          {
            /** Determine keyboard action **/

            switch (text[0])
            {
              /** Quit current panel or XPIP itself **/

              case 'Q':

                if (event_panel == Console->panels[0])
                {
                  for (i=1; i<Console->panel_count; i++)
                    close_panel (Console->panels[i]);
                  exit_routine (Console->panel);
                  XAutoRepeatOn (Win->display); 
                  XSync (Win->display, 0);
                  exit (0);
                }
                else
                {
                  close_panel (event_panel);
                }
                break;

              /** Force a redraw of the panel **/

              case 'R':
                draw_panel (event_panel);
                break;

              /** Edit a partially editable widget **/

              case 'P':
              case 'p':

                widget = handle_widget_search (event_panel);
                if (widget != NULL &&
			 widget->status[IS_EDITABLE] == EDITABLE_PARTIAL)
                  widget_function (widget, NULL, FUNCTION_EDIT); 
                widget = NULL;
                break;

              /** Dump current widget generic info **/

              case 'I':
                widget = strong_widget_search (event_panel,
					Win->mouse.x, Win->mouse.y);
                if (widget != NULL)
                  generic_dump (widget);
                widget = NULL;
                break;

              /** Dump current widget specific info **/

              case 'U':

                widget = strong_widget_search (event_panel,
					Win->mouse.x, Win->mouse.y);
                if (widget != NULL)
                  widget_function (widget, NULL, FUNCTION_DUMP); 
                widget = NULL;
                break;

/**
              case 'D':
              case 'd':
                dump_widget_info ("xpip.store");
                break;

              case 'U':
              case 'u':
                update_widget_info ("xpip.store");
                break;
**/
            }
          }
        }
        break;

      default:
        break;
    }

    /** Execute widget-specific function **/

    if (widget != NULL)
      widget_function (widget, NULL, FUNCTION_EXECUTE);

    /** Execute panel-specific routine **/

    if (use_execute)
      execute_routine (Console->panel, Win->event);

    /** If another panel has been specified, open it **/

    if (Console->next_panel != NULL)
    {
      Console->base_panel = Console->next_panel;;
      Console->next_panel = NULL;
      Console->panel = Console->base_panel;
      draw_panel (Console->panel, TRUE);
      Console->panel->is_drawn = TRUE;
      init_routine (Console->panel);
    }
  }
}



/******************************************************************************
  execute_routine (panel, event)

  Call the user-specfied routine to handle event-driven execution.
******************************************************************************/

execute_routine (panel, event)
  Panel *panel;
  XEvent event;
{
  int i, j;

#ifdef XPIP_USE_DIALOGS
  if (!strcmp (panel->name, "file_panel"))
    file_run ();
  else if (!strcmp (panel->name, "message_panel"))
    message_run ();
  else if (!strcmp (panel->name, "browse_panel"))
    browse_run ();
  else
    run_buffer (panel->index, event);
#else
  run_buffer (panel->index, event);
#endif
}



/******************************************************************************
  startup_routine (argc, argv)

  Call the user-specfied routine to handle program initialization.
******************************************************************************/

startup_routine (argc, argv)
  int *argc;
  char *argv[];
{
  startup_buffer (argc, argv);
}



/******************************************************************************
  init_routine (panel)

  Call the user-specfied routine to handle panel initiliazation.
******************************************************************************/

init_routine (panel)
  Panel *panel;
{
  init_buffer (panel->index);
}



/******************************************************************************
  exit_routine (panel)

  Call the user-specfied routine to handle panel exit.
******************************************************************************/

exit_routine (panel)
  Panel *panel;
{
  exit_buffer (panel->index);
}
