#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include "pip_constant.h"
#include "pip_extern.h"

#ifndef SEEK_SET
#define SEEK_SET	0
#endif


/******************************************************************************
  edit_text (widget)

  Manage the editing of the label of widget <widget>.
******************************************************************************/

edit_text (widget)
  Widget *widget;
{
  int state,
      width,
      length,
      done,
      textcount;

  char text[10],
       line[100];

  short data;

  KeySym key;

  set_widget_mode (widget, MODE_SELECT);

  /** Do type-specific editing setup **/

  strcpy (line, "_\0");
  widget_function (widget, line, FUNCTION_EDIT);

  /** Draw the updated widget **/

  draw_widget (widget);

  done = FALSE;

  /** Loop until enter **/

  while (!done)
  {
    /** Get event **/

    XNextEvent (Win->display, &(Win->event));

    /** Only worry about key presses **/

    if (Win->event.type == KeyPress)
      textcount = XLookupString((XKeyEvent *)(&(Win->event)),
                                        text, 10, &key, NULL);
    else 
      textcount = 0;
 
    if (textcount != 0)
    {
      text[1] = '_';
      text[2] = '\0';
      length = strlen (line);

      /** If return pressed **/

      if (text[0] == KEY_RETURN)
      {
        line[length-1] = '\0';
        done = TRUE;
        widget->reserve = EDIT_FINISH;
      }

      /** Else if backspace pressed **/

      else if (text[0] == KEY_BACKSPACE_1 || text[0] == KEY_BACKSPACE_2)
      {
        if (length > 1)
        {
          line[length-1] = '\0';
          line[length-2] = '_';
        }
      }

      /** Else normal key pressed **/

      else
      {
        line[length-1] = '\0';
        strcat (line, text); 
      }

      /** Perform type-specific update and redraw **/

      widget_function (widget, line, FUNCTION_EDIT);  
      draw_widget (widget);
      XSync (Win->display, 0);
    }
  }

  /** Flush events **/

  while (XEventsQueued (Win->display, QueuedAfterReading) > 0)
    XNextEvent (Win->display, &(Win->event));

  /** Unselect widget **/

  set_widget_mode (widget, MODE_NORMAL);
}


/******************************************************************************
  set_label (widget, line)

  Set the label pointer of <widget> to <line>.
******************************************************************************/

set_label (widget, line)
  Widget *widget;
  char *line;
{
  if (widget->label != NULL)
    free (widget->label);

  if (line == NULL)
    widget->label = NULL;
  else
    widget->label = strdup (line);
}



/******************************************************************************
  set_string (widget, line)

  Set the string pointer of <widget> to <line>.
******************************************************************************/

set_string (widget, line)
  Widget *widget;
  char *line;
{
  if (widget->string != NULL)
    free (widget->string);

  if (line == NULL)
    widget->string = NULL;
  else
    widget->string = strdup (line);
}



/******************************************************************************
  locate_text (widget, label, text_index, font_index, inside_widget)

  Given a series of formatting directions in <widget>, locate text in
  absolute and relative coordinates.
******************************************************************************/

locate_text (widget, label, text_index, font_index, inside_widget)
  Widget *widget;
  char *label;
  short text_index,
        font_index,
        inside_widget;
{
  XPoint temp;

  if (label == NULL)
    return;

  /** Center <label> within widget, both vertically and horizontally **/

  center_text (0, 0, widget->size.x, widget->size.y, label,
		&temp.x, &temp.y, widget->fonts[font_index]);

  /** Determine label offsets **/

  switch (widget->locate) 
  {
    case LABEL_NONE:
      return;

    case LABEL_GENERIC:
      temp.x = widget->locate_offset.x;
      temp.x = widget->locate_offset.y;
      break;

    case LABEL_NORTH:
    case LABEL_NORTH_WEST:
    case LABEL_NORTH_EAST:
      temp.y = 0;
      if (inside_widget)
        temp.y += Win->fonts[widget->fonts[font_index]]->dimension.y +
					widget->locate_offset.y;
      else
        temp.y -= widget->locate_offset.y;
      break;

    case LABEL_SOUTH:
    case LABEL_SOUTH_EAST:
    case LABEL_SOUTH_WEST:
      temp.y = widget->size.y;
      if (inside_widget)
        temp.y -= widget->locate_offset.y;
      else
        temp.y += Win->fonts[widget->fonts[font_index]]->dimension.y +
					widget->locate_offset.y;
      break;
  }

  /** Determine label offsets, this time moving text as necessary  **/

  switch (widget->locate) 
  {
    case LABEL_NORTH_WEST:
    case LABEL_SOUTH_WEST:
    case LABEL_WEST:
      temp.x = 0;
      if (inside_widget)
      {
        temp.x += widget->locate_offset.x;
        flush_text (label, &temp.x, widget->fonts[font_index], TRUE);
      }
      else
      {
        temp.x -= widget->locate_offset.x;
        flush_text (label, &temp.x, widget->fonts[font_index], FALSE);
      }
      break;

    case LABEL_NORTH_EAST:
    case LABEL_SOUTH_EAST:
    case LABEL_EAST:
      temp.x = widget->size.x;
      if (inside_widget)
      {
        temp.x -= widget->locate_offset.x;
        flush_text (label, &temp.x, widget->fonts[font_index], FALSE);
      }
      else
      {
        temp.x += widget->locate_offset.x;
        flush_text (label, &temp.x, widget->fonts[font_index], TRUE);
      }
      break;
  }

  /** Set label offsets **/

  widget->text_offset[text_index].x = temp.x;
  widget->text_offset[text_index].y = temp.y;
}



/******************************************************************************
  calculate_locations (panel)

  Determine widget locations for <panel>, given absolute and relative
  positioning specifications. 
******************************************************************************/

calculate_locations (panel)
  Panel *panel;
{
  int i,
      widget_index,
      space,
      count,
      done;
  char line[200];
  Widget *base_widgets[MAX_LINKS],
         *widget;

  /** Set base_widget size **/

  if (panel->base_widget->size.x == DIMENSION_PANEL)
    panel->base_widget->size.x = panel->size.x;
  if (panel->base_widget->size.y == DIMENSION_PANEL)
    panel->base_widget->size.y = panel->size.y;

  /** For each widget, perform panel centering if specified **/

  for (widget_index=0; widget_index < panel->widget_count; widget_index++)
  {
    widget = panel->widgets[widget_index];

    if (widget->position.x == LOCATE_CENTER)
      widget->position.x = Win->hint.width/2 - widget->size.x/2;
    if (widget->position.y == LOCATE_CENTER)
      widget->position.y = Win->hint.height/2 - widget->size.y/2;

    if (Master->use_v_2_1_definitions)
    {
      widget->delta.x = widget->spacing.x;
      widget->delta.y = widget->spacing.y;
    }
  }

  /** Keep repeating positioning process until nothing moves **/

  done = FALSE;
  for (count=0; count<panel->widget_count && !done; count++)
  {
    /** For each widget **/

    done = TRUE;
    for (widget_index=0; widget_index < panel->widget_count; widget_index++)
    {
      widget = panel->widgets[widget_index];

      /** If any aspect of position/size is not set **/

      if (widget->position.x < 0 || widget->position.y < 0 ||
			widget->size.x < 0 || widget->size.y < 0)
      {
        done = FALSE;

        /** For each nexus link possible **/

        for (i=0; i<MAX_LINKS; i++)
        {
          /** If no nexus name specified **/

          if (widget->link_names[i] == NULL)
          {
            /** If 0th link, use either next or previous widget as nexus **/

            if (i == 0)
            {
              if (widget_index == 0)
                base_widgets[i] = panel->widgets[widget_index + 1];
              else
                base_widgets[i] = panel->widgets[widget_index - 1];
            }
            else
              base_widgets[i] = base_widgets[LINK_BASIC];
          }
          else
            base_widgets[i] = get_named_widget (widget->link_names[i]);

          /** If no nexus, exit due to error **/

          if (base_widgets[i] == NULL)
          {
            sprintf (line, "widget \"%s\" linked to unknown widget \"%s\"",
		widget->name, widget->link_names[i]);
            throw_error (line, NULL);
          }

          /** Else if nexus same as widget, exit **/

          else if (base_widgets[i] == widget)
            throw_error ("widget \"%s\" linked to itself", widget->name);
        }

        /** If widget width not specified **/

        if (widget->size.x < 0)
        {
          if (widget->size.x == DIMENSION_TAFFY)
          {
            if (base_widgets[LINK_WIDTH]->size.x < 0 ||
				base_widgets[LINK_WIDTH]->position.x < 0)
              goto done_size_x;

            if (widget->position.x < 0)
              goto done_size_x;

            widget->size.x = base_widgets[LINK_WIDTH]->position.x +
				base_widgets[LINK_WIDTH]->size.x -
				widget->position.x + widget->delta.x;

            widget_function (widget, NULL, FUNCTION_SETUP);
          }
          else
          {
            if (base_widgets[LINK_WIDTH]->size.x < 0)
              goto done_size_x;

            widget->size.x = base_widgets[LINK_WIDTH]->size.x;

            /** necessary to recenter text **/

            widget_function (widget, NULL, FUNCTION_SETUP);
          }
        }

done_size_x:

        /** If widget height not specified **/

        if (widget->size.y < 0)
        {
          if (widget->size.y == DIMENSION_TAFFY)
          {
            if (base_widgets[LINK_HEIGHT]->size.y < 0 ||
				base_widgets[LINK_HEIGHT]->position.y < 0)
              goto done_size_y;

            if (widget->position.y < 0)
              goto done_size_y;

            widget->size.y = base_widgets[LINK_HEIGHT]->position.y +
				base_widgets[LINK_HEIGHT]->size.y -
				widget->position.y + widget->delta.y;

            widget_function (widget, NULL, FUNCTION_SETUP);
          }
          else
          {
            if (base_widgets[LINK_HEIGHT]->size.y < 0)
              goto done_size_y;

            widget->size.y = base_widgets[LINK_HEIGHT]->size.y;
            widget_function (widget, NULL, FUNCTION_SETUP);
          }
        }

done_size_y:

        /** If widget x position not specified **/

        if (widget->position.x < 0)
        {
          if (base_widgets[LINK_X]->position.x < 0)
            goto done_position_x;

          if ((widget->position.x == LOCATE_RIGHT ||
			widget->position.x == LOCATE_LEFT ||
			widget->position.x == LOCATE_CENTER ||
			widget->position.x == LOCATE_OVERLAY_CENTER) &&
				base_widgets[LINK_X]->size.x < 0)
            goto done_position_x;

          if ((widget->position.x == LOCATE_LEFT ||
			widget->position.x == LOCATE_OVERLAY_LEFT ||
			widget->position.x == LOCATE_OVERLAY_CENTER) &&
				widget->size.x < 0)
            goto done_position_x;

          space = widget->spacing.x;

          if (widget->position.x == LOCATE_SAME)
            widget->position.x = base_widgets[LINK_X]->position.x;
          else if (widget->position.x == LOCATE_RIGHT)
            widget->position.x = base_widgets[LINK_X]->position.x +
				base_widgets[LINK_X]->size.x + space;
          else if (widget->position.x == LOCATE_OVERLAY_RIGHT)
            widget->position.x = base_widgets[LINK_X]->position.x + space;
          else if (widget->position.x == LOCATE_LEFT)
            widget->position.x = base_widgets[LINK_X]->position.x -
					widget->size.x - space;
          else if (widget->position.x == LOCATE_OVERLAY_LEFT)
            widget->position.x = base_widgets[LINK_X]->position.x +
		base_widgets[LINK_X]->size.x - widget->size.x - space;
          else if (widget->position.x == LOCATE_OVERLAY_CENTER)
            widget->position.x = base_widgets[LINK_X]->position.x +
		base_widgets[LINK_X]->size.x/2 - widget->size.x/2;
        }

done_position_x:

        /** If widget y position not specified **/

        if (widget->position.y < 0)
        {
          if (base_widgets[LINK_Y]->position.y < 0)
            goto done_position_y;

          if ((widget->position.y == LOCATE_BELOW ||
			widget->position.y == LOCATE_ABOVE ||
			widget->position.y == LOCATE_CENTER ||
			widget->position.y == LOCATE_OVERLAY_CENTER) &&
				base_widgets[LINK_Y]->size.y < 0)
            goto done_position_y;

          if ((widget->position.y == LOCATE_ABOVE ||
			widget->position.y == LOCATE_OVERLAY_ABOVE ||
			widget->position.y == LOCATE_OVERLAY_CENTER) &&
				widget->size.y < 0)
            goto done_position_y;

          space = widget->spacing.y;

          if (widget->position.y == LOCATE_SAME)
            widget->position.y = base_widgets[LINK_Y]->position.y;
          else if (widget->position.y == LOCATE_BELOW)
            widget->position.y = base_widgets[LINK_Y]->position.y +
			base_widgets[LINK_Y]->size.y + space;
          else if (widget->position.y == LOCATE_OVERLAY_BELOW)
            widget->position.y = base_widgets[LINK_Y]->position.y + space;
          else if (widget->position.y == LOCATE_ABOVE)
            widget->position.y = base_widgets[LINK_Y]->position.y -
			widget->size.y - space;
          else if (widget->position.y == LOCATE_OVERLAY_ABOVE)
            widget->position.y = base_widgets[LINK_Y]->position.y +
			base_widgets[LINK_Y]->size.y -
			widget->size.y - space;
          else if (widget->position.y == LOCATE_OVERLAY_CENTER)
            widget->position.y = base_widgets[LINK_Y]->position.y +
			base_widgets[LINK_Y]->size.y/2 -
			widget->size.y/2;
        }

done_position_y:;
      }
    }
  }

  /** If not done, there is some type of circular linkage **/

  if (done == FALSE)
  {
    fprintf (stderr, "ERROR: unresolved or circular linkages\n");
    for (widget_index=0; widget_index<panel->widget_count; widget_index++)
    {
      widget = panel->widgets[widget_index];
      if (widget->position.x < 0 || widget->position.y < 0 ||
			widget->size.x < 0 || widget->size.y < 0)
        fprintf (stderr, "    %s  x: %2d  y: %2d  w: %2d  h: %2d\n",
		widget->name, widget->position.x, widget->position.y,
		widget->size.x, widget->size.y);
    }
    
    exit (0);
  }

  /** Set panel sizes **/

  panel->size.x = panel->base_widget->size.x;
  panel->size.y = panel->base_widget->size.y;
}



/******************************************************************************
  int
  get_text_width (string, font_index)

  Return width (in pixels) of <string> as printed in font <font_index>.
******************************************************************************/

int
get_text_width (string, font_index)
  char string[];
  int font_index;
{
  int junk;

  XCharStruct text_information; 

  XTextExtents (Win->fonts[font_index]->font, string, strlen (string),
			&junk, &junk, &junk, &text_information);
  return (text_information.rbearing - text_information.lbearing);
}


/******************************************************************************
  int
  get_text_height (string, font_index)

  Return height (in pixels) of <string> as printed in font <font_index>.
******************************************************************************/

int
get_text_height (string, font_index)
  char string[];
  int font_index;
{
  int junk;

  XCharStruct text_information; 

  XTextExtents (Win->fonts[font_index]->font, string, strlen (string),
			&junk, &junk, &junk, &text_information);
  return (text_information.ascent);
}



/******************************************************************************
  flush_text (string, xdest, font_index, flush_left)

  Flush the text of <string> either left (if <flush_left>) or right given
  pointer to desired horizontal location.
******************************************************************************/

flush_text (string, xdest, font_index, flush_left)
  char string[];
  short *xdest,
        font_index,
        flush_left;
{
  int junk;

  XCharStruct text_information; 

  if (string == NULL)
    return;

  XTextExtents (Win->fonts[font_index]->font, string, strlen (string),
			&junk, &junk, &junk, &text_information);

  if (flush_left)
    *xdest = *xdest - text_information.lbearing;
  else
    *xdest = *xdest - text_information.rbearing;
}



/******************************************************************************
  center_text (xloc, yloc, xsize, ysize, string, xdest, ydest, font_index)

  Center text of <string> in specified rectangle.
******************************************************************************/

center_text (xloc, yloc, xsize, ysize, string, xdest, ydest, font_index)
  short xloc, yloc,
        xsize, ysize,
        *xdest, *ydest,
        font_index;
  char string[];
{
  int junk;

  XCharStruct text_information; 

  if (string == NULL)
  {
    *xdest = xloc;
    *ydest = yloc;
    return;
  }

  XTextExtents (Win->fonts[font_index]->font, string, strlen (string),
			&junk, &junk, &junk, &text_information);

  *xdest = xloc + (xsize - (text_information.rbearing -
		text_information.lbearing))/2 - text_information.lbearing;

  *ydest = yloc + Win->fonts[font_index]->dimension.y +
		(ysize - Win->fonts[font_index]->dimension.y)/2;
}



/******************************************************************************
  load_graphics (widget)

  Load pixmaps and bitmaps for widget <widget>.
******************************************************************************/

load_graphics (widget)
  Widget *widget;
{
  int i, j,
      junk,
      status;

  /** Load each individual bitmap **/

  for (i=0; i<widget->graphic_count; i++)
  {
    if (widget->graphics[i]->is_bitmap)
    {
      status = XReadBitmapFile (Win->display, Win->window,
			widget->graphics[i]->filename,
			&(widget->graphics[i]->width),
			&(widget->graphics[i]->height),
			&(widget->graphics[i]->pixmap), &junk, &junk);
      if (status != 0) 
        fprintf (stderr, "WARNING: Bad bitmap %s (status %d)\n",
			widget->graphics[i]->filename, status);
    }
    else
    {
      load_pixmap (widget->graphics[i], widget);
    }
  }

  /** Set offsets for centering **/

  for (i=0; i<widget->graphic_count; i++)
  {
    widget->text_offset[i].x = (widget->size.x -
				widget->graphics[i]->width)/2;
    widget->text_offset[i].y = (widget->size.y -
				widget->graphics[i]->height)/2;
  }
}



/******************************************************************************
  load_pixmap (graphic)

  Load a full-color pixmap into the XPIP <graphic>, allocating new colors as
  necessary.
******************************************************************************/

load_pixmap (graphic, widget)
  Graphic *graphic;
  Widget *widget;
{
  if (strstr (graphic->filename, "bmp") == NULL)
    throw_error ("%s is not a bmp graphics file", graphic->filename);

  load_bmp (graphic->filename, graphic, widget);
}



/******************************************************************************
  load_bmp (filename, graphic, widget)

  Load a bmp format pixmap from file <filename> into XPIP <graphic> for
  widget <widget>.
******************************************************************************/

load_bmp (filename, graphic, widget)
  char *filename;
  Graphic *graphic;
  Widget *widget;
{
  int i, j, x, y,
      full_colors,
      good_colors,
      extra,
      bytes,
      bits,
      offset,
      bytes_in_width,
      bytes_in_scan,
      pixels_per_byte,
      width, height;

  unsigned char byte,
                fake_byte, mask,
                index,
                *byte_ptr,
                *image_ptr,
                byte_row[200],
                grid[100][100],
                triplet[3],
                mappings[256];

  FILE *fp;

  XImage *image;

  long junk,
       read_value();

  /** Open file, if possible **/

  if ((fp = fopen (filename, "r")) == NULL)
    throw_error ("Cannot open pixmap file %s", filename);

  /** Load header information **/

  junk =	read_value (2, fp);	/** type	**/
  junk =	read_value (4, fp);	/** size	**/
  junk =	read_value (2, fp);	/** rsv1	**/
  junk =	read_value (2, fp);	/** rsv2	**/
  offset =	read_value (4, fp);	/** offset	**/
  junk =	read_value (4, fp);	/** bisize	**/
  width =	read_value (4, fp);	/** width	**/
  height =	read_value (4, fp);	/** height	**/
  junk =	read_value (2, fp);	/** planes	**/
  bits =	read_value (2, fp);	/** bits	**/
  junk =	read_value (4, fp);	/** compress	**/
  junk =	read_value (4, fp);	/** size_im	**/
  junk =	read_value (4, fp);	/** xpel	**/
  junk =	read_value (4, fp);	/** ypel	**/
  full_colors =	read_value (4, fp);	/** colors	**/
  good_colors =	read_value (4, fp);	/** colors_imp	**/

  /** Load and assign colors **/

  for (i=0; i<good_colors; i++)
  {
    triplet[2] =	read_value (1, fp);	/** red		**/
    triplet[1] =	read_value (1, fp);	/** green	**/
    triplet[0] =	read_value (1, fp);	/** blue	**/
    junk =		read_value (1, fp);	/** dunno	**/

    mappings[i] = get_color (triplet);
  }

  for (; i<full_colors; i++)
  {
    junk =		read_value (1, fp);	/** red		**/
    junk =		read_value (1, fp);	/** green	**/
    junk =		read_value (1, fp);	/** blue	**/
    junk =		read_value (1, fp);	/** dunno	**/
  }

  /** If transparent, reassign 0th color mapping **/

  if (graphic->gc)
    mappings[0] = Win->color_widget[widget->gcs[GC_BODY]].pixel; 

  /** Determine how packing is done **/

  if (bits == 0)
    bits = 1;

  if (pixels_per_byte == 0)
    pixels_per_byte = 1;

  pixels_per_byte = 8/bits;
  bytes_in_width = width/pixels_per_byte;
  if (width % pixels_per_byte != 0)
    bytes_in_width += 1;

  if (bytes_in_width % 4 == 0)
    bytes_in_scan = bytes_in_width;
  else
    bytes_in_scan = bytes_in_width + (4-bytes_in_width%4);

  /** Create image for storage **/

  image = XCreateImage (Win->display,
		DefaultVisual (Win->display, Win->screen), Win->depth,
		ZPixmap, 0, 0, width, height, 8, 0);
  image->data = (char *)(malloc(image->bytes_per_line*height));

  mask = 1;
  for (i=0; i<bits; i++)
    mask*=2;
  mask -= 1;

  /** Skip to the start of the pixmap **/

  fseek (fp, (long)(offset), SEEK_SET);

  /** Load pixmap **/

  for (y=0; y<height; y++)
  {
    image_ptr = (unsigned char *)(image->data) +
		image->bytes_per_line*height - (y+1)*image->bytes_per_line;

    fread (byte_row, bytes_in_scan, 1, fp);
    byte_ptr = byte_row;

    for (x=0,i=0; i<bytes_in_width; i++)
    {
      for (j=0; j<pixels_per_byte && x<width; j++)
      {
        fake_byte = (*byte_ptr) >> (bits*(pixels_per_byte-j-1));
        index = fake_byte & mask;
        *image_ptr++ = mappings[index];
        x++;
      }
      byte_ptr++;
    }
  }

  fclose (fp);

  /** Allocate pixmap and copy over image **/

  graphic->pixmap = XCreatePixmap (Win->display, Win->window,
			width, height, Win->depth);
  graphic->width = width;
  graphic->height = height;

  XPutImage (Win->display, graphic->pixmap, Win->hue_widget[0], image,
			0, 0, 0, 0, width, height);

  /** Free image storage **/

  free (image->data);
  free (image);
}



/******************************************************************************
  long
  read_value (count, fp)

  Read a value <count> bytes from file <fp>, using reversed significance.
******************************************************************************/

long
read_value (count, fp)
  int count;
  FILE *fp;
{
  long i,
       value,
       factor;
  unsigned char bytes[8];

  for (i=0; i<count; i++)
    fread (&bytes[i], 1, 1, fp);

  value = 0;
  factor = 1;
  for (i=0; i<count; i++)
  {
    value += bytes[i]*factor;
    factor *= 256;
  }

  return (value);
}
