#include <stdio.h>
#include <math.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

/** Mandatory include browses **/

#include "pip_constant.h"
#include "pip_extern.h"

/******************************************************************************
  browse_run ()

  Handle analysis of browse_panel widgets.
******************************************************************************/

browse_run ()
{
  check_browse_widgets ();
}


/******************************************************************************
  check_browse_widgets ()

  Check the browse panel widgets for user control, updating display when
  necessary.
******************************************************************************/

check_browse_widgets ()
{
  int i;
  Widget *widget;

  /** Free up list widget text **/

  if (check_named_widget_flag (&widget, "browse_ok_button"))
  {
    widget = get_named_widget ("browse_list");
    list_load (widget, NULL, 0, FALSE, FALSE);

    close_panel (Console->event_panel);
  }

/**
  if (check_named_widget_flag (&widget, "browse_print_button"))
  {
    widget = get_named_widget ("browse_list_text");
    for (i=0; i<widget->state_count; i++)
      printf ("%s\n", widget->superlabels[i]);
    fflush (stdout);
  }
**/

  reset_all_flags ();
}



/******************************************************************************
  browse_show_list (label, strings, count)

  Display a generic list in the browse panel.
******************************************************************************/

browse_show_list (label, strings, count)
  char *label,
       *strings[];
  int count;
{
  int i, j,
      limit;

  char full_string[500],
       first_string[500],
       second_string[500];

  Widget *widget;

  set_active_panel (get_named_panel ("browse_panel"));

  widget = get_named_widget ("browse_list");
  list_load (widget, strings, count, FALSE, FALSE);
  draw_named_widget ("browse_list_text");

  if (label == NULL)
    set_named_widget_label ("browse_header_data", " ");
  else
    set_named_widget_label ("browse_header_data", label);

  restore_panel ();

  open_panel (get_named_panel ("browse_panel"));
}



/******************************************************************************
  browse_show_file (label, directory, filename)

******************************************************************************/

browse_show_file (label, directory, filename)
  char *label,
       *directory,
       *filename;
{
  int i, j,
      count;

  char *strings[10000],
       full_filename[1000],
       line[1000];

  FILE *fp;

  if (directory == NULL)
    sprintf (full_filename, "%s", filename);
  else if (filename == NULL)
    sprintf (full_filename, "%s", directory);
  else
    sprintf (full_filename, "%s%s", directory, filename);

  if ((fp = fopen (full_filename, "r")) == NULL)
  {
    throw_visual_error ("Cannot browse file %s", full_filename);
    return;
  }

  count = 0;
  while (fgets (line, 1000, fp))
  {
    line[strlen(line)-1] = '\0';
    strings[count++] = strdup (line);
    if (count >= 10000)
      break;
  }

  browse_show_list (label, strings, count);

  for (i=0; i<count; i++)
    free (strings[i]);
}



/******************************************************************************
  browse_init ()
******************************************************************************/

browse_init ()
{
}



/******************************************************************************
  browse_exit ()
******************************************************************************/

browse_exit ()
{
  Widget *widget;

  widget = get_named_widget ("browse_list");
  list_load (widget, NULL, 0, FALSE, FALSE);
}
