#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

/** Mandatory include files **/

#include "pip_constant.h"
#include "pip_extern.h"

/** Application include files **/

font_run ()
{
  check_font_widgets ();
}


check_font_widgets ()
{
  int index;
  char string[400];
  Widget *widget;

  if (check_named_widget_flag (&widget, "font_list_text"))
  {
    list_get_entry (widget, string, -1);
    set_named_widget_label ("font_data", string);

    font_set (string);
  }

  if (check_named_widget_flag (&widget, "ok_button"))
    close_panel (Console->event_panel);

  if (check_named_widget_flag (&widget, "reload_button"))
  {
    font_init ();
  }

  if (check_named_widget_flag (&widget, "load_button"))
  {
    if (widget->state == 1)
    {
      file_init_widgets ("fonts/", "*.fnt");
      file_set_callback_widget (widget);
      open_panel (get_named_panel ("file_panel"));
    }
    else if (widget->state == 2)
    {
      if (widget->user)
      {  
        file_get_filename (string, NULL, NULL);
        font_load_list (string);
      }  

      set_widget_state (widget, 0);
    }
  }

  if (check_named_widget_flag (&widget, "drop_button"))
  {
    widget = get_named_widget ("font_list_text");
    list_get_entry (widget, string, -1);
    printf ("%s\n", string);
    fflush (stdout);
  }

  reset_all_flags ();
}



font_init ()
{
  int i,
      count;
  char **list,
       font_name[200],
       font_foundry[100],
       font_family[100],
       font_pixel[10],
       font_weight[10],
       font_slant[10],
       font_spacing[10];

  Panel *panel;
  Widget *widget;

  set_active_panel (get_named_panel ("font_panel"));

  widget = get_named_widget ("foundry_data");
  strcpy (font_foundry, widget->label);

  widget = get_named_widget ("family_data");
  strcpy (font_family, widget->label);

  widget = get_named_widget ("pixel_data");
  if (widget->value < 1)
    strcpy (font_pixel, "*");
  else
    sprintf (font_pixel, "%d", (int)(widget->value));

  widget = get_named_widget ("weight_button");
  switch (widget->state)
  {
    case 0:
      strcpy (font_weight, "normal");
      break;
    case 1:
      strcpy (font_weight, "bold");
      break;
    case 2:
      strcpy (font_weight, "medium");
      break;
    case 3:
      strcpy (font_weight, "*");
      break;
  }
  
  widget = get_named_widget ("slant_button");
  switch (widget->state)
  {
    case 0:
      strcpy (font_slant, "r");
      break;
    case 1:
      strcpy (font_slant, "i");
      break;
    case 2:
      strcpy (font_slant, "o");
      break;
    case 3:
      strcpy (font_slant, "*");
      break;
  }

  widget = get_named_widget ("spacing_button");
  switch (widget->state)
  {
    case 0:
      strcpy (font_spacing, "p");
      break;
    case 1:
      strcpy (font_spacing, "m");
      break;
    case 2:
      strcpy (font_spacing, "c");
      break;
    case 3:
      strcpy (font_spacing, "*");
      break;
  }

  if (font_foundry[0] == '*' && font_family[0] == '*' &&
		font_weight[0] == '*' && font_slant[0] == '*' &&
		font_pixel[0] == '*' && font_spacing[0] == '*')
    strcpy (font_name, "*");
  else
    sprintf (font_name, "*-%s-%s-%s-%s-*-*-%s-*-*-*-%s-*-*-*",
		font_foundry, font_family, font_weight,
		font_slant, font_pixel, font_spacing);

  list = XListFonts (Win->display, font_name, 1000, &count);
  list_load (get_named_widget ("font_list"), list, count, TRUE, FALSE);

  set_named_widget_label ("font_data", Xip->text->fontname);

  XFreeFontNames (list);

  restore_panel ();
}



font_set (fontname)
  char *fontname;
{
  int junk;
  XCharStruct text;

  /** ALTER: get rid of this **/

  int char_limit;
  XTextItem line_information[2];
  char *strdup();
 

  XFreeFont (Win->display, Xip->text->font);

  strcpy (Xip->text->fontname, fontname);

  Xip->text->font = XLoadQueryFont (Win->display, Xip->text->fontname);
  if (Xip->text->font == 0)
  {
    throw_warning ("Font %s not found", Xip->text->fontname);
    Xip->text->font = XQueryFont (Win->display,
                XGContextFromGC(DefaultGC(Win->display,Win->screen)));
  }

  XSetFont (Win->display, Win->hue_usable[0], Xip->text->font->fid);
  XSetFont (Win->display, Win->hue_usable[1], Xip->text->font->fid);

  XTextExtents (Xip->text->font, "T", 1, &junk, &junk, &junk, &text);

  Xip->text->dimension.x = text.width;
  Xip->text->dimension.y = text.ascent;

  if (2*text.width > Xip->text->size.x || 2*text.ascent > Xip->text->size.y)
  {
    free (Xip->text->ximage->data);
    XDestroyImage (Xip->text->ximage);

    XFreePixmap (Win->display, Xip->text->pixmap);

    Xip->text->size.x = 2*text.width;
    Xip->text->size.y = 2*text.ascent;

    Xip->text->pixmap = XCreatePixmap (Win->display, Win->window,
                        Xip->text->size.x, Xip->text->size.y, Win->depth);
    Xip->text->ximage = XCreateImage (Win->display,
                        DefaultVisual (Win->display, Win->screen),
                        Win->depth,
                        ZPixmap, 0, 0,
                        Xip->text->size.x, Xip->text->size.y,
                        8, 0);
    Xip->text->ximage->data = (char *)
                        (malloc(Xip->text->size.x * Xip->text->size.y));
  }

  /** ALTER: get rid of this eventually **/

  XFillRectangle (Win->display, Win->window,
                        Win->hue_usable[Xip->background_index],
                        0, 0,
                        400, 100);

  line_information[0].chars = strdup ("Moose are good 0123456789");
  line_information[0].nchars = strlen ("Moose are good 0123456789");
 
  line_information[0].font = Xip->text->font->fid;
  line_information[0].delta = 0;
 
  XDrawText (Win->display, Win->window, Win->hue_usable[1],
			20, 70, line_information, 1);
 
  free (line_information[0].chars);

  XSync (Win->display, Win->window);
}


font_load_list (filename)
  char *filename;
{
  int i,
      count;
  char line[500],
       *strings[1000];
  FILE *fp;

  if (filename == NULL)
    return;

  if ((fp = fopen (filename, "r")) == NULL)
  {
    throw_warning ("Cannot open font file %s", filename);
    return;
  }

  count = 0;
  while (fgets (line, 500, fp))
  {
    if (line[0] == '#')
      continue;

    line[strlen(line)-1] = '\0';

    strings[count++] = strdup (line);
  }

  fclose (fp);

  list_load (get_named_widget ("font_list"), strings, count, TRUE, FALSE);

  for (i=0; i<count; i++) 
    free (strings[i]);
}
