#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

/** Mandatory includes **/

#include "pip_constant.h"
#include "pip_extern.h"

/******************************************************************************
  message_run ()

  Handle analysis of message_panel widgets.
******************************************************************************/

message_run ()
{
  check_message_widgets ();
}


/******************************************************************************
  check_message_widgets ()

  Check the message panel widgets for user control, updating display when
  necessary.
******************************************************************************/

check_message_widgets ()
{
  Panel *panel;

  Widget *widget;

  if (check_named_widget_flag (&widget, "ok_button"))
    close_panel (Console->event_panel);

  reset_all_flags ();
}



/******************************************************************************
  message_show_error (string)
******************************************************************************/

message_show_error (string)
  char *string;
{
  throw_visual_error (string, NULL);
}



/******************************************************************************
  message_show_warning (string)
******************************************************************************/

message_show_warning (string)
  char *string;
{
  throw_visual_warning (string, NULL);
}



/******************************************************************************
  message_show (title, string)
******************************************************************************/

message_show (title, string)
  char *string;
{
  throw_visual_message (title, string, NULL);
}



/******************************************************************************
  throw_visual_error (widget_string, parameter_string)

  Print an error message to the error panel.
******************************************************************************/

throw_visual_error (widget_string, parameter_string)
  char *widget_string,
       *parameter_string;
{
  throw_visual_message ("ERROR!", widget_string, parameter_string);
}



/******************************************************************************
  throw_visual_warning (widget_string, parameter_string)

  Print a warning message to the error panel.
******************************************************************************/

throw_visual_warning (widget_string, parameter_string)
  char *widget_string,
       *parameter_string;
{
  throw_visual_message ("WARNING!", widget_string, parameter_string);
}



/******************************************************************************
  throw_visual_message (type_string, widget_string, parameter_string)

  Print a generic message to the error panel.
******************************************************************************/

throw_visual_message (type_string, widget_string, parameter_string)
  char *type_string,
       *widget_string,
       *parameter_string;
{
  int i, j,
      limit;

  char full_string[500],
       first_string[500],
       second_string[500];

  Widget *widget;

  set_active_panel (get_named_panel ("message_panel"));

  set_named_widget_label ("header_data", type_string);

  if (parameter_string == NULL)
    sprintf (full_string, "%s\n", widget_string);
  else
    sprintf (full_string, widget_string, parameter_string);

  widget = get_named_widget ("message_1_data");

  /** Determine maximum number of characters on one line **/

  limit = widget->size.x / (Win->fonts[widget->fonts[0]]->dimension.x);
  if (limit >= MAX_LABEL_LENGTH)
    limit = MAX_LABEL_LENGTH - 1;

  strcpy (first_string, full_string);
  strcpy (second_string, "\0");

  for (i=0; i<limit && full_string[i] != '\0'; i++)
    first_string[i] = full_string[i];
  first_string[i] = '\0';

  /** If string doesn't fit on first line, determine where to break and	**/
  /** assign remainder to second line.					**/

  if (i == limit)
  {
    j = i;
    for (; full_string[j] != ' ' && full_string[j] != '\t' && j != 0; j--);

    if (j != 0)
    {
      first_string[j] = '\0';
      i = j+1;
    }

    strcpy (second_string, &full_string[i]);
  }
   
  set_named_widget_label ("message_1_data", first_string);
  set_named_widget_label ("message_2_data", second_string);

  restore_panel ();

  open_panel (get_named_panel ("message_panel"));
}



/******************************************************************************
  message_init ()
******************************************************************************/

message_init ()
{
}



/******************************************************************************
  message_exit ()
******************************************************************************/

message_exit ()
{
}
